#! /usr/bin/env python

import openturns as ot

ot.TESTPREAMBLE()

model = ot.SymbolicFunction(["x0", "x1", "x2", "x3"], ["-(6+x0^2-x1+x2+3*x3)"])
dim = model.getInputDimension()
marginals = [ot.Normal(5.0, 3.0) for i in range(dim)]
distribution = ot.JointDistribution(
    marginals, ot.BlockIndependentCopula([ot.ClaytonCopula(), ot.NormalCopula()])
)
# distribution = ot.Normal([5]*dim, [3]*dim, ot.CorrelationMatrix(dim))
# distribution = ot.JointDistribution(marginals, ot.IndependentCopula(dim))
distribution.setDescription(["marginal" + str(i) for i in range(dim)])
vect = ot.RandomVector(distribution)
output = ot.CompositeRandomVector(model, vect)
event = ot.ThresholdEvent(output, ot.Greater(), 0.0)
solver = ot.Cobyla()
solver.setMaximumCallsNumber(1000)
solver.setMaximumAbsoluteError(1.0e-10)
solver.setMaximumRelativeError(1.0e-10)
solver.setMaximumResidualError(1.0e-10)
solver.setMaximumConstraintError(1.0e-10)
solver.setStartingPoint(distribution.getMean())
algo = ot.FORM(solver, event)
algo.run()
result = algo.getResult()
hasoferReliabilityIndexSensitivity = result.getHasoferReliabilityIndexSensitivity()
print(hasoferReliabilityIndexSensitivity)



"""
Mixture of experts
==================
"""

# %%
# In this example we are going to approximate a piece wise continuous function using an expert mixture of metamodels.
#
# The metamodels will be represented by the family of functions :math:`f_k \forall \in [1, n_c]`:
#
# .. math::
#      f(\vect{x}) = f_k(\vect{x})
#
# for any :math:`\vect{z} \in \textrm{Class}_k` where the :math:`n_c \in \Nset` classes are defined by the classifier.
#
# Using the supervised mode the classifier partitions the input and output spaces at once:
#
# .. math::
#    \vect{z} = (\vect{x}, f(\vect{x}))
#
# The classifier is :class:`~openturns.MixtureClassifier` based on a :class:`~openturns.Mixture` distribution defined as:
#
# .. math::
#    p(\vect{x}) = \sum_{i=1}^{n_c} w_i p_i(\vect{x})
#
#
# The rule to assign a point to a class is defined as follows: :math:`\vect{x}` is assigned to the class :math:`j = \operatorname{argmax}_j \log w_k p_k(\vect{z})`.
#
# The grade of :math:`\vect{x}` with respect to the class :math:`k` is :math:`\log w_k p_k(\vect{x})`.
#
#

# %%
import openturns as ot
import openturns.viewer as otv
import numpy as np


# %%
dimension = 1

# Define the piecewise model we want to rebuild


def piecewise(X):
    # if x < 0.0:
    #     f = (x+0.75)**2-0.75**2
    # else:
    #     f = 2.0-x**2
    xarray = np.asarray(X)
    return np.piecewise(
        xarray,
        [xarray < 0, xarray >= 0],
        [lambda x: x * (x + 1.5), lambda x: 2.0 - x * x],
    )


f = ot.PythonFunction(1, 1, func_sample=piecewise)

# %%
# Build a metamodel over each segment
degree = 5
samplingSize = 100
enumerateFunction = ot.LinearEnumerateFunction(dimension)
productBasis = ot.OrthogonalProductPolynomialFactory(
    [ot.LegendreFactory()] * dimension, enumerateFunction
)
adaptiveStrategy = ot.FixedStrategy(
    productBasis, enumerateFunction.getStrataCumulatedCardinal(degree)
)

# %%
# Interval 1: :math:`[-1.0; 0.0]`
d1 = ot.Uniform(-1.0, 0.0)
X1 = d1.getSample(samplingSize)
Y1 = f(X1)
fc1 = ot.FunctionalChaosAlgorithm(X1, Y1, d1, adaptiveStrategy)
fc1.run()
mm1 = fc1.getResult().getMetaModel()
graph = mm1.draw(-1.0, -1e-6)
view = otv.View(graph)

# %%
# Interval 2: :math:`[0.0, 1.0]`
d2 = ot.Uniform(0.0, 1.0)
X2 = d2.getSample(samplingSize)
Y2 = f(X2)
fc2 = ot.FunctionalChaosAlgorithm(X2, Y2, d2, adaptiveStrategy)
fc2.run()
mm2 = fc2.getResult().getMetaModel()
graph = mm2.draw(1e-6, 1.0)
view = otv.View(graph)

# %%
# Define the mixture
R = ot.CorrelationMatrix(2)
d1 = ot.Normal([-1.0, -1.0], [1.0] * 2, R)  # segment 1
d2 = ot.Normal([1.0, 1.0], [1.0] * 2, R)  # segment 2
weights = [1.0] * 2
atoms = [d1, d2]
mixture = ot.Mixture(atoms, weights)

# %%
# Create the classifier based on the mixture
classifier = ot.MixtureClassifier(mixture)

# %%
# Create local experts using the metamodels
experts = ot.Basis([mm1, mm2])

# %%
# Create a mixture of experts
evaluation = ot.ExpertMixture(experts, classifier)
moe = ot.Function(evaluation)

# %%
# Draw the mixture of experts
graph = moe.draw(-1.0, 1.0)
view = otv.View(graph)

# %%
# Display all figures
otv.View.ShowAll()



#! /usr/bin/env python

import openturns as ot

ot.TESTPREAMBLE()


# Default constructor
myDefaultModel = ot.UserDefinedStationaryCovarianceModel()
print("myDefaultModel = ", myDefaultModel)

# Default dimension parameter to evaluate the model
dimension = 1
inputDimension = 1

# Amplitude values
amplitude = ot.Point(dimension)
# Scale values
scale = ot.Point(dimension)
# Spatial correclation
spatialCorrelation = ot.CorrelationMatrix(dimension)
for index in range(dimension):
    # constant amplitude
    amplitude[index] = 2.0
    scale[index] = (index + 1.0) / dimension

# Sample a ExponentialModel
referenceModel = ot.ExponentialModel(scale, amplitude, spatialCorrelation)

print("reference model=", referenceModel)
size = 100
timeGrid = ot.RegularGrid(0.0, 0.1, size)
covarianceCollection = ot.SquareMatrixCollection(size)

for i in range(timeGrid.getN()):
    t = timeGrid.getValue(i)
    covarianceCollection[i] = referenceModel(t)
# Create a UserDefinedStationaryCovarianceModel
myModel = ot.UserDefinedStationaryCovarianceModel(timeGrid, covarianceCollection)
print("myModel=", myModel)

for i in range(timeGrid.getN()):
    t = timeGrid.getValue(i)
    # We look for cov(s,t) ==> when adding to the collection, we compute cov(t,s)
    # Because of symmetry, we check the right index computation
    print(
        "t= %.6g" % t,
        "myModel =  %.6g" % myModel(t)[0, 0],
        ", referenceModel=  %.6g" % referenceModel(t)[0, 0],
    )

# Test the drawing method as a nonstationary model, in the covariance range
graph = myModel.draw(0, 0, -2.0, 2.0, 21, True, False)
print(graph)
# Test the drawing method as a nonstationary model, in the correlation
# range
graph = myModel.draw(0, 0, -2.0, 2.0, 21, True, True)
print(graph)



"""
Quick start guide to distributions
==================================
"""

# %%
# Abstract
# ---------
#
# In this example, we present classes for univariate and multivariate distributions.
# We demonstrate the probabilistic programming capabilities of the library.
# For univariate distributions, we show how to compute the probability density function (if any),
# the cumulative distribution function and the quantiles.
# We also show how to create graphics.
# The :class:`~openturns.JointDistribution` class, which creates a distribution based on its
# marginals and its copula, is presented.
# We show how to truncate any distribution with the :class:`~openturns.TruncatedDistribution` class.

# %%
# Univariate distribution
# -----------------------
#
# The library is a probabilistic programming library: it is possible to create a random variable and perform operations on this variable *without* generating a sample.
#
# Several *univariate distributions* are implemented in the library. The most commonly used are:
#
# - :class:`~openturns.Uniform`,
# - :class:`~openturns.Normal`,
# - :class:`~openturns.Beta`,
# - :class:`~openturns.LogNormal`,
# - :class:`~openturns.Exponential`,
# - :class:`~openturns.WeibullMin`.

# %%
import openturns as ot
import openturns.viewer as otv

# %%
# The uniform distribution
# ------------------------
#
# Let us create a uniform random variable :math:`\mathcal{U}(2,5)`.

# %%
uniform = ot.Uniform(2, 5)

# %%
# The :meth:`~openturns.Distribution.drawPDF` method plots the probability density function.

# %%
graph = uniform.drawPDF()
view = otv.View(graph)

# %%
# The :meth:`~openturns.Distribution.computePDF` method computes the probability distribution at a specific point.

# %%
uniform.computePDF(3.5)

# %%
# The :meth:`~openturns.Distribution.drawCDF` method plots the cumulative distribution function.

# %%
graph = uniform.drawCDF()
view = otv.View(graph)

# %%
# The :meth:`~openturns.Distribution.computeCDF` method computes the value of the cumulative distribution function a given point.

# %%
uniform.computeCDF(3.5)

# %%
# The :meth:`~openturns.Distribution.getSample` method generates a sample.

# %%
sample = uniform.getSample(10)
sample

# %%
# The most common way to "see" a sample is to plot the empirical histogram.

# %%
sample = uniform.getSample(1000)
graph = ot.HistogramFactory().build(sample).drawPDF()
view = otv.View(graph)

# %%
# Multivariate distributions with or without independent copula
# -------------------------------------------------------------

# %%
# We can create multivariate distributions by two different methods:
#
# - we can also create a multivariate distribution by combining a list of univariate marginal distribution and a copula,
# - some distributions are defined as multivariate distributions: :class:`~openturns.Normal`, :class:`~openturns.Dirichlet`, :class:`~openturns.Student`.

# %%
# Define a multivariate Normal distribution in dimension 4

# %%
distribution = ot.Normal(4)
distribution

# %%
# Since the method based on a marginal and a copula is more flexible, we illustrate below this principle.
# In the following script, we define a bivariate distribution made of two univariate distributions (Gaussian and uniform) and an independent copula.
# The second input argument of the :class:`~openturns.JointDistribution` class is optional: if it is not specified, the copula is independent by default.

# %%
normal = ot.Normal()
uniform = ot.Uniform()
distribution = ot.JointDistribution([normal, uniform])
distribution

# %%
# We can also use the :class:`~openturns.IndependentCopula` class.

# %%
normal = ot.Normal()
uniform = ot.Uniform()
copula = ot.IndependentCopula(2)
distribution = ot.JointDistribution([normal, uniform], copula)
distribution

# %%
# We see that this produces the same result: in the end of this section, we will change the copula and see what happens.

# %%
# The :meth:`~openturns.Distribution.getSample` method produces a sample from this distribution.

# %%
distribution.getSample(10)

# %%
# In order to visualize a bivariate sample, we can use the :class:`~openturns.Cloud` class.

# %%
sample = distribution.getSample(1000)
showAxes = True
graph = ot.Graph("X0~N, X1~U", "X0", "X1", showAxes)
cloud = ot.Cloud(sample, "blue", "fsquare", "")  # Create the cloud
graph.add(cloud)  # Then, add it to the graph
view = otv.View(graph)

# %%
# We see that the marginals are Gaussian and uniform and that the copula is independent.

# %%
# Define a plot a copula
# ----------------------

# %%
# The :class:`~openturns.NormalCopula` class allows one to create a Gaussian copula. Such a copula is defined by its correlation matrix.

# %%
R = ot.CorrelationMatrix(2)
R[0, 1] = 0.6
copula = ot.NormalCopula(R)
copula

# %%
# We can draw the contours of a copula with the :meth:`~openturns.NormalCopula.drawPDF` method.

# %%
graph = copula.drawPDF()
view = otv.View(graph)

# %%
# Multivariate distribution with arbitrary copula
# -----------------------------------------------

# %%
# Now that we know that we can define a copula, we create a bivariate distribution with normal and uniform marginals and an arbitrary copula.
# We select the Ali-Mikhail-Haq copula as an example of a non trivial dependence.

# %%
normal = ot.Normal()
uniform = ot.Uniform()
theta = 0.9
copula = ot.AliMikhailHaqCopula(theta)
distribution = ot.JointDistribution([normal, uniform], copula)
distribution

# %%
sample = distribution.getSample(1000)
showAxes = True
graph = ot.Graph("X0~N, X1~U, Ali-Mikhail-Haq copula", "X0", "X1", showAxes)
cloud = ot.Cloud(sample, "blue", "fsquare", "")  # Create the cloud
graph.add(cloud)  # Then, add it to the graph
view = otv.View(graph)

# %%
# We see that the sample is quite different from the previous sample with independent copula.

# %%
# Draw several distributions in the same plot
# -------------------------------------------

# %%
# It is sometimes convenient to create a plot presenting the PDF and CDF on the same graphics. This is possible thanks to Matplotlib.

# %%
beta = ot.Beta(5, 7, 9, 10)
pdfbeta = beta.drawPDF()
cdfbeta = beta.drawCDF()
exponential = ot.Exponential(3)
pdfexp = exponential.drawPDF()
cdfexp = exponential.drawCDF()

# %%

# %% slideshow={"slide_type": "subslide"}
grid = ot.GridLayout(2, 2)
grid.setGraph(0, 0, pdfbeta)
grid.setGraph(0, 1, cdfbeta)
grid.setGraph(1, 0, pdfexp)
grid.setGraph(1, 1, cdfexp)
view = otv.View(grid)

# %%
# Truncate a distribution
# -----------------------
#
# Any distribution can be truncated with the :class:`~openturns.TruncatedDistribution` class.
# Let :math:`f_X` (resp. :math:`F_X`) the PDF (resp. the CDF) of the real
# random variable :math:`X`. Let :math:`a` and :math:`b` two reals with :math:`a<b`.
# Let :math:`Y` be the random variable defined by:
#
# .. math::
#     Y = \max(a, \min(b, X)).
#
# Its distribution is the distribution of :math:`X` truncated to the :math:`[a,b]` interval.
# Therefore, the PDF of :math:`Y` is:
#
# .. math::
#    f_Y(y) = \frac{f_X(y)}{F_X(b) - F_X(a)}
#
#
# if :math:`y\in[a,b]` and :math:`f_Y(y)=0` otherwise.

# %%
# Consider for example the log-Normal variable :math:`X` with mean :math:`\mu=0` and standard deviation :math:`\sigma=1`.

# %%
X = ot.LogNormal()
graph = X.drawPDF()
view = otv.View(graph)

# %%
# We can truncate this distribution to the :math:`[1,2]` interval. We see that the PDF of the distribution becomes discontinuous at the truncation points 1 and 2.

# %%
Y = ot.TruncatedDistribution(X, 1.0, 2.0)
graph = Y.drawPDF()
view = otv.View(graph)

# %%
# We can also also truncate it with only a lower bound.

# %%
Y = ot.TruncatedDistribution(X, 1.0, ot.TruncatedDistribution.LOWER)
graph = Y.drawPDF()
view = otv.View(graph)

# %%
# We can finally truncate a distribution with an upper bound.

# %%
Y = ot.TruncatedDistribution(X, 2.0, ot.TruncatedDistribution.UPPER)
graph = Y.drawPDF()
view = otv.View(graph)

# %%
# In the specific case of the Gaussian distribution, the specialized :class:`~openturns.TruncatedNormal` distribution can be used
# instead of the generic :class:`~openturns.TruncatedDistribution` class.

# %%
# Display all figures
otv.View.ShowAll()



#! /usr/bin/env python

import openturns as ot
import openturns.testing as ott

ot.TESTPREAMBLE()

ot.RandomGenerator.SetSeed(0)

# Instantiate one distribution object
dim = 2
copula = ot.GalambosCopula(0.5)
ott.assert_almost_equal(copula.getMean(), [0.5, 0.5], 0.0, 0.0)
ref_cov_matrix = ot.CovarianceMatrix(2, [0.0833333, 0.0241064, 0.0241064, 0.0833333])
ott.assert_almost_equal(copula.getCovariance(), ref_cov_matrix, 1e-5, 0.0)

# Is this copula an elliptical distribution?
assert not copula.isElliptical()

# Is this copula elliptical?
assert not copula.hasEllipticalCopula()

# Is this copula independent?
assert not copula.hasIndependentCopula()

# Test for realization of distribution
ott.assert_almost_equal(copula.getRealization(), [0.629877, 0.871724], 1e-5, 0.0)

# Test correlation and covariance
ott.assert_almost_equal(
    copula.getCorrelation(),
    ot.CovarianceMatrix(2, [1, 0.289277, 0.289277, 1]),
    1e-5,
    0.0,
)
ott.assert_almost_equal(
    copula.getCovariance(),
    ot.CovarianceMatrix(2, [0.0833333, 0.0241064, 0.0241064, 0.0833333]),
    1e-5,
    0.0,
)
ott.assert_almost_equal(
    copula.getSpearmanCorrelation(),
    ot.CovarianceMatrix(2, [1, 0.289277, 0.289277, 1]),
    1e-5,
    0.0,
)
ott.assert_almost_equal(
    copula.getKendallTau(),
    ot.CovarianceMatrix(2, [1, 0.19643, 0.19643, 1]),
    1e-5,
    0.0,
)

# Test for sampling
oneSample = copula.getSample(10)
ref_sample = [
    [0.135276, 0.0205545],
    [0.347057, 0.937166],
    [0.92068, 0.699802],
    [0.0632061, 0.188369],
    [0.714382, 0.437797],
    [0.373767, 0.67536],
    [0.883503, 0.441284],
    [0.92851, 0.925177],
    [0.684575, 0.834214],
    [0.359802, 0.916859],
]
ott.assert_almost_equal(oneSample, ref_sample, 1e-5, 0.0)

# Test for sampling
anotherSample = copula.getSample(10000)
ott.assert_almost_equal(anotherSample.computeMean(), [0.502777, 0.501261], 1e-5, 0.0)
ref_cov_matrix_2 = ot.CovarianceMatrix(2, [0.0833333, 0.0241064, 0.0244522, 0.0833333])
ott.assert_almost_equal(copula.getCovariance(), ref_cov_matrix_2, 1e-5, 0.0)

# Define a point
point = ot.Point(dim, 0.2)

# Show PDF and CDF of point
ott.assert_almost_equal(copula.computePDF(point), 1.231981, 1e-5, 0.0)
ott.assert_almost_equal(copula.computeCDF(point), 0.059814, 1e-4, 0.0)

# Get 50% quantile
quantile = copula.computeQuantile(0.5)
ott.assert_almost_equal(quantile, [0.67295, 0.67295], 1e-4, 0.0)
ott.assert_almost_equal(copula.computeCDF(quantile), 0.500000, 1e-4, 0.0)

# Get 5% quantile
quantile = copula.computeQuantile(0.95, True)
ott.assert_almost_equal(quantile, [0.180532, 0.180532], 1e-5, 0.0)

# Get 95% survival function
inverseSurvival = ot.Point(copula.computeInverseSurvivalFunction(0.95))
ott.assert_almost_equal(inverseSurvival, [0.0258323, 0.0258323], 1e-4, 0.0)
ott.assert_almost_equal(
    copula.computeSurvivalFunction(inverseSurvival), 0.950000, 1e-4, 0.0
)
ott.assert_almost_equal(copula.computeEntropy(), -0.062336, 1e-4, 0.0)

# Test conditional methods
y = [0.5]
x = 0.6
p = copula.computeConditionalCDF(x, y)
ott.assert_almost_equal(p, 0.62707, 1e-4, 0.0)
ott.assert_almost_equal(
    copula.computeConditionalQuantile(p, y), 0.5999999999999999, 1e-4, 0.0
)

# Extract the marginals
margin_0 = copula.getMarginal(0)
ott.assert_almost_equal(margin_0.computePDF([0.25]), 1.0, 0.0, 0.0)
ott.assert_almost_equal(margin_0.computeCDF([0.25]), 0.25, 0.0, 0.0)
ott.assert_almost_equal(margin_0.computeQuantile(0.95), [0.95], 0.0, 0.0)
ott.assert_almost_equal(margin_0.getRealization(), [0.56939], 1e-5, 0.0)
margin_1 = copula.getMarginal(1)
ott.assert_almost_equal(margin_1.computePDF([0.25]), 1.0, 0.0, 0.0)
ott.assert_almost_equal(margin_1.computeCDF([0.25]), 0.25, 0.0, 0.0)
ott.assert_almost_equal(margin_1.computeQuantile(0.95), [0.95], 0.0, 0.0)
ott.assert_almost_equal(margin_1.getRealization(), [0.0571768], 1e-5, 0.0)

# Extract a 2-D marginal
margins = copula.getMarginal([1, 0])
ott.assert_almost_equal(margins.computePDF([0.25] * 2), 1.1784, 1e-4, 0.0)
ott.assert_almost_equal(margins.computeCDF([0.25] * 2), 0.0883883, 1e-5, 0.0)
quantile = ot.Point(margins.computeQuantile(0.95))
ott.assert_almost_equal(quantile, [0.971115, 0.971115], 1e-5, 0.0)
ott.assert_almost_equal(margins.computeCDF(quantile), 0.950000, 1e-4, 0.0)
ott.assert_almost_equal(margins.getRealization(), [0.915787, 0.243714], 1e-5, 0.0)

ot.Log.Show(ot.Log.TRACE)
validation = ott.DistributionValidation(copula)
validation.setMomentsSamplingSize(100000)
validation.run()



import openturns as ot
from matplotlib import pyplot as plt
import openturns.viewer as otv

pdf_graph = ot.Graph("PDF graph", "x", "PDF")
pdf_graph.setLegendPosition("upper right")
cdf_graph = ot.Graph("CDF graph", "x", "CDF")
cdf_graph.setLegendPosition("lower right")
palette = ot.Drawable.BuildDefaultPalette(10)
for i, v in enumerate([(2, 0.83), (4, 0.66), (20, 0.33)]):
    r, p = v
    distribution = ot.Polya(r, p)
    pdf_curve = distribution.drawPDF().getDrawable(0)
    cdf_curve = distribution.drawCDF().getDrawable(0)
    pdf_curve.setColor(palette[i])
    cdf_curve.setColor(palette[i])
    legend = f"r={r}, p={p}"
    pdf_curve.setLegend(legend)
    cdf_curve.setLegend(legend)
    pdf_graph.add(pdf_curve)
    cdf_graph.add(cdf_curve)
fig = plt.figure(figsize=(10, 4))
pdf_axis = fig.add_subplot(121)
cdf_axis = fig.add_subplot(122)
otv.View(pdf_graph, figure=fig, axes=[pdf_axis], add_legend=True)
otv.View(cdf_graph, figure=fig, axes=[cdf_axis], add_legend=True)
fig.suptitle("Polya(r,p)")



#! /usr/bin/env python

import openturns as ot
import openturns.testing as ott

ot.TESTPREAMBLE()

ot.ResourceMap.SetAsUnsignedInteger(
    "OptimizationAlgorithm-DefaultMaximumIterationNumber", 1000
)
ot.ResourceMap.SetAsUnsignedInteger(
    "OptimizationAlgorithm-DefaultMaximumCallsNumber", 100000
)
ot.ResourceMap.SetAsScalar("OptimizationAlgorithm-DefaultMaximumAbsoluteError", 1.0e-7)
ot.ResourceMap.SetAsScalar("OptimizationAlgorithm-DefaultMaximumRelativeError", 1.0e-7)
ot.ResourceMap.SetAsScalar("OptimizationAlgorithm-DefaultMaximumResidualError", 1.0e-7)
ot.ResourceMap.SetAsScalar(
    "OptimizationAlgorithm-DefaultMaximumConstraintError", 1.0e-7
)
ot.PlatformInfo.SetNumericalPrecision(2)

# The 1D mesher
mesher1D = ot.LevelSetMesher([7])
print("mesher1D=", mesher1D)

level = 0.5
function1D = ot.SymbolicFunction("x", "cos(x)/(1+0.1*x^2)")
levelSet1D = ot.LevelSet(function1D, ot.LessOrEqual(), level)

# Manual bounding box
mesh1D = mesher1D.build(levelSet1D, ot.Interval(-10.0, 10.0))
print("mesh1D=", mesh1D)

# The 2D mesher
mesher2D = ot.LevelSetMesher([5] * 2)
print("mesher2D=", mesher2D)

function2D = ot.SymbolicFunction(
    ["x0", "x1"], ["cos(x0 * x1)/(1 + 0.1 * (x0^2 + x1^2))"]
)
levelSet2D = ot.LevelSet(function2D, ot.LessOrEqual(), level)

# Manual bounding box, linear interpolation
mesh2D = mesher2D.build(levelSet2D, ot.Interval([-10.0] * 2, [10.0] * 2), False)
print("mesh2D=", mesh2D)

# Manual bounding box, solve the equation projection
ot.ResourceMap.SetAsBool("LevelSetMesher-SolveEquation", True)
mesh2D = mesher2D.build(levelSet2D, ot.Interval([-10.0] * 2, [10.0] * 2), True)
print("mesh2D=", mesh2D)

# Manual bounding box, optimization projection
ot.ResourceMap.SetAsBool("LevelSetMesher-SolveEquation", False)
mesh2D = mesher2D.build(levelSet2D, ot.Interval([-10.0] * 2, [10.0] * 2), True)
print("mesh2D=", mesh2D)

# The 3D mesher
mesher3D = ot.LevelSetMesher([3] * 3)
print("mesher3D=", mesher3D)

function3D = ot.SymbolicFunction(
    ["x0", "x1", "x2"], ["cos(x0 * x1 + x2)/(1 + 0.1*(x0^2 + x1^2 + x2^2))"]
)
levelSet3D = ot.LevelSet(function3D, ot.LessOrEqual(), level)

# Manual bounding box
mesh3D = mesher3D.build(levelSet3D, ot.Interval([-10.0] * 3, [10.0] * 3))
print("mesh3D=", mesh3D)

# The 4D mesher
mesher4D = ot.LevelSetMesher([3] * 4)
print("mesher4D=", mesher4D)

function4D = ot.SymbolicFunction(
    ["x0", "x1", "x2", "x3"], ["sqrt(x0^2+x1^2+x2^2+x3^2)"]
)
levelSet4D = ot.LevelSet(function4D, ot.LessOrEqual(), level)

# Manual bounding box
mesh4D = mesher4D.build(levelSet4D, ot.Interval([-0.5] * 4, [0.5] * 4), False)
print("mesh4D=", mesh4D)

# Issue #1668
f = ot.SymbolicFunction(["x", "y"], ["x^2+y^2"])
levelset = ot.LevelSet(f, ot.Less(), 1.0)
mesh = ot.LevelSetMesher([16] * 2).build(levelset, ot.Interval([-1.5] * 2, [1.5] * 2))
gLess = mesh.draw()
f = ot.SymbolicFunction(["x", "y"], ["-(x^2+y^2)"])
levelset = ot.LevelSet(f, ot.Greater(), -1.0)
mesh = ot.LevelSetMesher([16] * 2).build(levelset, ot.Interval([-1.5] * 2, [1.5] * 2))
gGreater = mesh.draw()
ott.assert_almost_equal(
    gLess.getDrawable(0).getData(), gGreater.getDrawable(0).getData(), 1e-4, 1e-4
)



#! /usr/bin/env python

import openturns as ot

ot.TESTPREAMBLE()

# Dimension of the input model
# Size of the TimeGrid
# dimension parameter
dimension = 1

# Amplitude values
amplitude = ot.Point(dimension, 1.00)

# Scale values
scale = ot.Point(dimension, 1.0)

size = 10
timeGrid = ot.RegularGrid(0.0, 0.1, size)

# Absolute model
model = ot.AbsoluteExponential(scale, amplitude)
myProcess = ot.GaussianProcess(model, timeGrid)

# Create a Process sample of size N
N = 10000
sample = myProcess.getSample(N)

# NonStationaryCovarianceModelFactory using default parameter - Factory
# initiate
myFactory = ot.NonStationaryCovarianceModelFactory()

# Build a UserDefinedCovarianceModel using the Welch method
myCovarianceModel = myFactory.buildAsUserDefinedCovarianceModel(sample)

for i in range(size):
    t = timeGrid.getValue(i)
    for j in range(size):
        s = timeGrid.getValue(j)
        estimatedValue = myCovarianceModel(t, s)[0, 0]
        modelValue = model(t, s)[0, 0]
        print(
            "Covariance C( %.6g" % t,
            ",  %.6g" % s,
            ") : ",
            " evaluation =  %.6g" % estimatedValue,
            " model =  %.6g" % modelValue,
        )



"""
Visualize sensitivity
=====================
"""

# %%
# The parallel coordinates graph enables to visualize all the combinations of the input
# variables which lead to a specific range of the output variable. It is a very simple and cheap tool to visualize sensitivity from the raw data.
#
# Let us consider a model :math:`f: \mathbb{R}^n \longrightarrow \mathbb{R}`, where :math:`f(\vect{X}) = Y`.
#
# The graph requires to have an input sample :math:`X_s` and an output sample :math:`Y_s`.
#
# The first figure draws such a graph: each column represents one component
# :math:`X_i` of the input vector :math:`\vect{X}`.
# The last column represents the scalar output variable :math:`Y`.
# For each point :math:`\vect{X}^j`, each component :math:`X_i^j`
# is noted on its respective axe and the last mark is the one which corresponds to
# the associated :math:`Y^j`. A line joins all the marks. Thus, each point of the sample
# corresponds to a particular line on the graph.
#
# The scale of the axes are quantile based: each axe runs between 0 and 1 and each
# value is represented by its quantile with respect to its marginal empirical distribution.
#
# It is interesting to select, among those lines, the ones which correspond to a specific range of the output variable.
# These particular lines are colored differently.
# This specific range is defined in the quantile based scale of :math:`Y` or in its specific scale.
# In that second case, the range is automatically converted into a quantile based scale range.

# %%
from openturns.usecases.ishigami_function import IshigamiModel
import openturns as ot
import openturns.viewer as otv

# %%
# We load the :ref:`Ishigami model<use-case-ishigami>` from the `usecases` module :
im = IshigamiModel()
# the Ishigami function
model = im.model
# the input distribution
inputDist = im.distribution

# %%
# We create a random vector from out input distribution :
inputVector = ot.RandomVector(inputDist)

# %%
# And we create the output random vector :math:`Y = model(X)` :
output = ot.CompositeRandomVector(model, inputVector)

# %%
# We generate an input sample of size :math:`N` :
N = 1000
X = inputVector.getSample(N)

# %%
# We evaluate the associated output sample :
Y = model(X)
Y.setDescription("Y")

# %%
# We display the minimum, maximum and value of the 90% quantile of :math:`Y` :
print(Y.getMin(), Y.getMax(), Y.computeQuantilePerComponent(0.9))

# %%
# Value based scale to describe the Y range
# -----------------------------------------
#
# Say we are interested in the higher values of the output :math:`Y`. A first
# approach is to highlight peculiar lines for which :math:`Y \in [a,b]` with
# the bounds :math:`a` and :math:`b` well chosen. For example, values greater
# than 85% of the maximum :
#
#  - :math:`a = 0.85 \max(Y)` ;
#  - :math:`b = \max(Y)` ;
#
minValue = 0.85 * Y.getMax()[0]
maxValue = Y.getMax()[0]

# We deactivate the default quantile scale.
quantileScale = False
graph = ot.VisualTest.DrawParallelCoordinates(
    X, Y, minValue, maxValue, "red", quantileScale
)
graph.setLegendPosition("lower right")
view = otv.View(graph)

# %%
# Here we would like to conclude that the highest values of :math:`Y` are obtained from a specific input as the highlighted lines clearly follow one only path.
# However, this approach is too naive and specific to the input sample. Indeed,
# if we set the lower bound to 80% of the maximum :

# %%
minValue = 0.80 * Y.getMax()[0]
maxValue = Y.getMax()[0]
quantileScale = False
graph = ot.VisualTest.DrawParallelCoordinates(
    X, Y, minValue, maxValue, "red", quantileScale
)
graph.setLegendPosition("lower right")
view = otv.View(graph)

# %%
# A new path is then available ! That is the reason why we chose a quantile based ranking as the value based parallel plot involves a bit of guessing.


# %%
# Rank based scale to describe the Y range
# ---------------------------------------------------
#
# In this paragraph we use quantile based bounds. We are still interested in
# the highest values of :math:`Y` more specifically the 95% quantile :

# %%
minValue = 0.95
maxValue = 1.0
# a quantileScale is used, default behaviour
quantileScale = True
graph = ot.VisualTest.DrawParallelCoordinates(
    X, Y, minValue, maxValue, "red", quantileScale
)
graph.setLegendPosition("lower right")
view = otv.View(graph)

# %%
# The parallel coordinates plot obtained is helpful : we see peculiar values for each marginal.
#

# %%
# Recall that the Ishigami model is given by
#
# .. math::
#    f(X) = \sin(X_1) + 7 \sin^2(X_2) + 0.1 X_3^4 \sin(X_1)
#
# with each marginal of :math:`X=(X_1, X_2, X_3)` uniform in :math:`]-\pi,\pi[`.
#
# Then the highest values of :math:`Y=f(X)` are obtained when each term is near its peak :
#
# - the :math:`\sin(X_1)` term around :math:`X_1 = \frac{\pi}{2}` ;
# - the :math:`7 \sin^2(X_2)` term around :math:`X_2 = -\frac{\pi}{2}` and :math:`X_2 = \frac{\pi}{2}` ;
# - the :math:`X_3^4 \sin(X_1)` term around :math:`X_1 = \frac{\pi}{2}` and :math:`X_3 = \{ -\pi, \pi \}`.
#
# These values can be seen on the parallel plot as for each marginal there is a
# cluster around respectively 1, 2 and 2 values for :math:`X_1`, :math:`X_2` and :math:`X_3`.
# This amounts to 4 different values 'realizing' the maximum and we can observe
# 4 distinct paths on the parallel plot as well.
#

# %%
# We can also guess the independence of marginals when looking at paths between
# :math:`X_2` and :math:`X_3`. For any given cluster value of :math:`X_2` on the
# graph there are as many paths to a high value of :math:`X_3` as to a small value.
# A dependence between these two marginals would have presented unbalanced paths.

# %%
# When the parallel plot brings nothing
# -------------------------------------
#
# To conclude our tour on the parallel plot we look at the 50% quantile : that is
# values around the mean :

# %%
minValue = 0.48
maxValue = 0.52
quantileScale = True
graph = ot.VisualTest.DrawParallelCoordinates(
    X, Y, minValue, maxValue, "red", quantileScale
)
graph.setLegendPosition("upper right")
view = otv.View(graph)

# %%
# We cannot extract any useful information from this parallel plot.
# In fact it is the expected behaviour as mean values should be attained from
# various combinations of# the input variables.
# The parallel coordinates graph is a cheap tool and highly useful to explore
# more extreme values!

# %%
# Display figures
otv.View.ShowAll()



#! /usr/bin/env python

import openturns as ot

ot.TESTPREAMBLE()
ot.PlatformInfo.SetNumericalPrecision(3)

distribution = ot.Pareto(2.0, 3.5, -1.0)
size = 10000
sample = distribution.getSample(size)
sample.exportToCSVFile("Pareto.csv")
factory = ot.ParetoFactory()
print("distribution=", repr(distribution))
print("Estimated distribution (Moments)=", factory.buildMethodOfMoments(sample))
print(
    "Estimated distribution (MLE)=", factory.buildMethodOfLikelihoodMaximization(sample)
)
print("Estimated distribution (LSQ)=", factory.buildMethodOfLeastSquares(sample))
estimatedDistribution = factory.build()
print("Default distribution=", estimatedDistribution)
estimatedDistribution = factory.build(distribution.getParameter())
print("Distribution from parameters=", estimatedDistribution)
estimatedPareto = factory.buildAsPareto(sample)
print("Estimated pareto=", estimatedPareto)
estimatedPareto = factory.buildAsPareto()
print("Default pareto=", estimatedPareto)
estimatedPareto = factory.buildAsPareto(distribution.getParameter())
print("Pareto from parameters=", estimatedPareto)



"""
Use case : cantilever beam
==========================
"""

import openturns as ot


class CantileverBeam:
    """
    Data class for the cantilever beam example.


    Attributes
    ----------

    dim : int
        The dimension of the problem, dim=4.

    E : :class:`~openturns.Beta`
        Young's modulus distribution
        Beta(0.9, 3.5, 65.0e9, 75.0e9)

    F : :class:`~openturns.LogNormal`
        Load distribution
        LogNormalMuSigma()([300.0, 30.0, 0.0])

    L : :class:`~openturns.Uniform`
        Length distribution
        Uniform(2.5, 2.6)

    II : :class:`~openturns.Beta`
        Moment of inertia distribution
        Beta(2.5, 4.0, 1.3e-7, 1.7e-7)

    model : :class:`~openturns.SymbolicFunction`
        The physical model of the cantilever beam.

    R : :class:`~openturns.CorrelationMatrix`
        Correlation matrix used to define the copula.

    copula : :class:`~openturns.NormalCopula`
        Copula of the model.

    distribution : :class:`~openturns.JointDistribution`
        The joint distribution of the parameters.

    independentDistribution : :class:`~openturns.JointDistribution`
        The joint distribution of the parameters with independent copula.

    data : :class:`~openturns.Sample`
        Sample of the input distribution

    Examples
    --------
    >>> from openturns.usecases import cantilever_beam
    >>> # Load the cantilever beam model
    >>> cb = cantilever_beam.CantileverBeam()
    """

    def __init__(self):
        self.dim = 4  # number of inputs
        # Young's modulus E
        self.E = ot.Beta(0.9, 3.5, 65.0e9, 75.0e9)  # in N/m^2
        self.E.setDescription("E")
        self.E.setName("Young modulus")

        # Load F
        self.F = ot.LogNormal()  # in N
        self.F.setParameter(ot.LogNormalMuSigma()([300.0, 30.0, 0.0]))
        self.F.setDescription("F")
        self.F.setName("Load")

        # Length L
        self.L = ot.Uniform(2.5, 2.6)  # in m
        self.L.setDescription("L")
        self.L.setName("Length")

        # Moment of inertia I
        self.II = ot.Beta(2.5, 4.0, 1.3e-7, 1.7e-7)  # in m^4
        self.II.setDescription("I")
        self.II.setName("Inertia")

        # physical model
        self.model = ot.SymbolicFunction(["E", "F", "L", "I"], ["F*L^3/(3*E*I)"])

        # correlation matrix
        self.R = ot.CorrelationMatrix(self.dim)
        self.R[2, 3] = -0.2
        self.copula = ot.NormalCopula(
            ot.NormalCopula.GetCorrelationFromSpearmanCorrelation(self.R)
        )
        self.distribution = ot.JointDistribution(
            [self.E, self.F, self.L, self.II], self.copula
        )

        # special case of an independent copula
        self.independentDistribution = ot.JointDistribution(
            [self.E, self.F, self.L, self.II]
        )

        self.data = self.distribution.getSample(100)



"""
Create a random design of experiments
=====================================
"""

# %%
# Abstract
# --------
#
# Random designs of experiments can be generated from probability distributions.

# %%
import openturns as ot
import openturns.viewer as otv

# %%
# We create the underlying distribution: a standard 2-dimensional normal distribution.
distribution = ot.Normal(2)
size = 50

# %%
# The Monte Carlo design of experiments
# -------------------------------------
#
# We build the experiment  with the :class:`~openturns.MonteCarloExperiment` class :
experiment = ot.MonteCarloExperiment(distribution, size)
sample = experiment.generate()

# %%
# We draw the design of experiments as a :class:`~openturns.Cloud`
graph = ot.Graph("Monte Carlo design", r"$x_1$", r"$x_2$")
cloud = ot.Cloud(sample, "blue", "fsquare", "")
graph.add(cloud)
view = otv.View(graph)

# %%
# Latin Hypercube Sampling
# ------------------------
#
# We build the LHS design of experiments with the :class:`~openturns.LHSExperiment` class :
experiment = ot.LHSExperiment(distribution, size)
sample = experiment.generate()

# %%
# We draw the LHS design of experiments as a cloud :
graph = ot.Graph("LHS design", r"$x_1$", r"$x_2$")
cloud = ot.Cloud(sample, "blue", "fsquare", "")
graph.add(cloud)
view = otv.View(graph)

# %%
# Display all figures
otv.View.ShowAll()



import math as m
import openturns as ot


def _exec(X):
    wab, wcd, t, P, L, rho, g, a, b = X
    wab = wab / 1000
    wcd = wcd / 1000
    t = t / 1000
    L = L / 1000
    P = P * 1000
    try:
        MB = 1 / 3 * (P * L) + (rho * g * wcd * t * (L**a)) / 18
        Masse = rho * t * L * (2 / (3 * m.sin(b)) * wab + wcd)
        return [MB, Masse]
    except OverflowError:
        return [1e30] * 2


prior = ot.JointDistribution([ot.Normal(0.1, 0.01)] * 3)
model = ot.PythonFunction(9, 2, _exec)
model_p = ot.ParametricFunction(model, [6, 7, 8], prior.getMean())

obs_sample = ot.Sample.ImportFromCSVFile("t_Calibration_console.csv", ",")
obs_in = obs_sample.getMarginal([0, 1, 2, 3, 4, 5])
obs_out = obs_sample.getMarginal([10, 11])
err_cov = ot.CovarianceMatrix(2)
for i in range(2):
    err_cov[i, i] = 0.1**2
p0cov = prior.getCovariance()
p0 = prior.getMean()
algos = [
    ot.GaussianNonLinearCalibration(model_p, obs_in, obs_out, p0, p0cov, err_cov),
    ot.NonLinearLeastSquaresCalibration(model_p, obs_in, obs_out, p0),
]
for algo in algos:
    names = []
    if ot.PlatformInfo.HasFeature("cminpack"):
        names.append("CMinpack")
    if ot.PlatformInfo.HasFeature("ceres"):
        names.append("LEVENBERG_MARQUARDT")

    for name in names:
        solver = ot.OptimizationAlgorithm.GetByName(name)
        algo.setOptimizationAlgorithm(solver)
        algo.setBootstrapSize(0)
        algo.run()
        result = algo.getResult()
        theta = result.getParameterMAP()
        print("theta=", theta)

        posterior = result.getParameterPosterior()
        sample = posterior.getSample(100000)
        level = 0.95
        lb = sample.computeQuantilePerComponent((1 - level) / 2)
        ub = sample.computeQuantilePerComponent(1 - (1 - level) / 2)
        ci = ot.Interval(lb, ub)
        print("ci=", ci)
        assert theta in ci



#! /usr/bin/env python

import openturns as ot

ot.TESTPREAMBLE()

matrix1 = ot.IdentityMatrix(2)
print("matrix1 = ", repr(matrix1))

pt = ot.Point()
pt.add(5.0)
pt.add(0.0)
print("pt = ", repr(pt))

result = ot.Point()
result = matrix1.solveLinearSystem(pt)
print("result = ", repr(result))

determinant = matrix1.computeDeterminant()
print("determinant = %.6f" % determinant)

ev = ot.ScalarCollection(2)
ev = matrix1.computeEigenValues()
print("ev = ", repr(ev))

if matrix1.isPositiveDefinite():
    isSPD = "true"
else:
    isSPD = "false"
print("isSPD = ", isSPD)

matrix2 = matrix1.computeCholesky()
print("matrix2 = ", repr(matrix2))

det, sgn = matrix1.computeLogAbsoluteDeterminant()
print(det)
assert det == 0.0, "wrong det"

sv = matrix1.computeSingularValues()
print(sv)
assert sv == [1.0] * 2, "wrong sv"

ev, u, vT = matrix1.computeSVD()
print(ev, u, vT)
assert ev == [1.0] * 2, "wrong svd"

ev, v = matrix1.computeEV()
print(ev, v)
assert ev == [1.0] * 2, "wrong ev"

tr = matrix1.transpose()
print(tr)
assert tr == matrix1, "transpose"

cov = ot.CovarianceMatrix(2)
prod = matrix1 @ cov
print(prod)
assert prod == matrix1, "product /cov"



#! /usr/bin/env python

import openturns as ot
import openturns.testing as ott

ot.TESTPREAMBLE()


def fitKriging(covarianceModel):
    """
    Fit the parameters of a kriging metamodel.
    """
    coordinates = ot.Sample(
        [
            [1.0, 1.0],
            [5.0, 1.0],
            [9.0, 1.0],
            [1.0, 3.5],
            [5.0, 3.5],
            [9.0, 3.5],
            [1.0, 6.0],
            [5.0, 6.0],
            [9.0, 6.0],
        ]
    )
    observations = ot.Sample(
        [[25.0], [25.0], [10.0], [20.0], [25.0], [20.0], [15.0], [25.0], [25.0]]
    )
    basis = ot.ConstantBasisFactory(2).build()
    algo = ot.KrigingAlgorithm(coordinates, observations, covarianceModel, basis)
    algo.run()
    krigingResult = algo.getResult()
    return krigingResult


# Isotropic covariance model
myIsotropicKernel = ot.IsotropicCovarianceModel(ot.SquaredExponential(), 2)
krigingFittedCovarianceModel = fitKriging(myIsotropicKernel).getCovarianceModel()
ott.assert_almost_equal(krigingFittedCovarianceModel.getScale()[0], 2.86427, 0.0, 1e-4)
ott.assert_almost_equal(
    krigingFittedCovarianceModel.getAmplitude()[0], 6.65231, 0.0, 1e-4
)

with ott.assert_raises(TypeError):
    ot.IsotropicCovarianceModel(ot.SquaredExponential(), 0)



#! /usr/bin/env python
"""
@author: Michaël Baudin (EDF), Antoine Dumas (Phiméca)

Analyses the convergence of the standard deviation of the asymptotic
distribution of the Sobol' index estimator depending on the
sample size.
"""


import openturns as ot
from gsobollib import gsobolDistribution, gsobol
from ishigamilib import (
    ishigamiDistribution,
    ishigamiGSymbolic,
)
from numpy import sqrt, array
import numpy as np
import matplotlib as mpl

import pylab as pl
from sobol_variance_estimators import (
    SaltelliSensitivityAlgorithm,
    MartinezSensitivityAlgorithm,
    JansenSensitivityAlgorithm,
    MauntzKucherenkoSensitivityAlgorithm,
)

mpl.use("Agg")  # to remove to show figure
ot.Log.Show(ot.Log.ERROR)


def computeVariance(model, distribution, sobol_estimator, sampleSize):
    """
    Compute the asymptotic standard deviation of the Sobol' indices estimator.

    Parameters
    ----------
    model : ot.Function
        The function.
    distribution : ot.Distribution
        The input distribution.
    sobol_estimator : ot.SobolSensitivityAlgorithm
        The estimator.
    sampleSize : int
        The sample size.

    Returns
    -------
    std_first_asymptotic : ot.Point(dimension)
        The componentwise standard deviation of the first order index.
    std_total_asymptotic : ot.Point(dimension)
        The componentwise standard deviation of the total order index.

    """

    sobolexperiment = ot.SobolIndicesExperiment(distribution, int(sampleSize), False)
    inputDesign = sobolexperiment.generate()
    outputDesign = model(inputDesign)
    sensitivity_algorithm = sobol_estimator(inputDesign, outputDesign, int(sampleSize))

    # Récupère les distributions asymptotiques
    distFirstCol = sensitivity_algorithm.getFirstOrderAsymptoticDistribution()
    distTotalCol = sensitivity_algorithm.getTotalOrderAsymptoticDistribution()
    std_first_asymptotic = ot.Point([d.getStandardDeviation()[0] for d in distFirstCol])
    std_total_asymptotic = ot.Point([d.getStandardDeviation()[0] for d in distTotalCol])

    return std_first_asymptotic, std_total_asymptotic


def plotVariance(
    algorithmName, list_sampleSize, list_std_first_asymptotic, list_std_total_asymptotic
):
    """
    Plots the standard deviation depending on the sample size.

    Each figure is saved into "Convergence_Variance-" + algorithmName + ".png".

    Parameters
    ----------
    algorithmName : str
        The name of the test case.
    list_sampleSize : list of ints
        The list of sample sizes.
    list_std_first_asymptotic : list of floats
        The list of standard deviations of the first order indices.
    list_std_total_asymptotic : TYPE
        The list of standard deviations of the total order indices.

    Returns
    -------
    None.

    """
    fig, ax = pl.subplots()
    ax.set_title(algorithmName)
    ax.plot(list_sampleSize, 1.0 / sqrt(list_sampleSize), "--", label="$1/\\sqrt{n}$")

    for i in range(list_std_first_asymptotic.getDimension()):
        ax.plot(
            list_sampleSize,
            np.hstack(list_std_first_asymptotic[:, i]),
            "o",
            label="$S_{%i}$" % i,
        )
        ax.plot(
            list_sampleSize,
            np.hstack(list_std_total_asymptotic[:, i]),
            "o",
            label="$T_{%i}$" % i,
        )
    ax.set_xlabel("N")
    ax.set_ylabel("Variance")
    ax.set_xscale("log")
    ax.set_yscale("log")
    ax.legend(bbox_to_anchor=(1.0, 1.0))
    fig.show()
    fig.savefig(
        "Convergence_stddev-" + algorithmName + ".png",
        transparent=True,
        bbox_inches="tight",
    )
    return None


def runConvergence(
    model,
    distribution,
    sobol_estimator,
    list_sampleSize,
):
    """
    Performs the convergence analysis.

    Parameters
    ----------
    model : ot.Function
        The function.
    distribution : ot.Distribution
        The input distribution.
    sobol_estimator : ot.SobolSensitivityAlgorithm
        The estimator.
    list_sampleSize : list of ints
        The list of sample sizes.

    Returns
    -------
    None.

    """

    list_std_first_asymptotic = ot.Sample(
        len(list_sampleSize), distribution.getDimension()
    )
    list_std_total_asymptotic = ot.Sample(
        len(list_sampleSize), distribution.getDimension()
    )
    for i, sampleSize in enumerate(list_sampleSize):
        # Compute errors
        list_std_first_asymptotic[i], list_std_total_asymptotic[i] = computeVariance(
            model, distribution, sobol_estimator, sampleSize
        )

    algorithmName = "%s - %s" % (model.getName(), sobol_estimator.__name__)
    plotVariance(
        algorithmName,
        list_sampleSize,
        list_std_first_asymptotic,
        list_std_total_asymptotic,
    )


if __name__ == "__main__":
    # Sizes of simulation
    n_loops = 10
    list_sampleSize_float = np.logspace(1, 5, n_loops)  # n_loops valeurs de 1e1 à 1e5
    list_sampleSize = [int(v) for v in list_sampleSize_float]

    ref_sampleSize = 4e5
    ref_nrepetitions = 1000

    # GSOBOL

    a = array([0, 9, 99])
    # Distribution uniforme associée au cas-test GSobol
    distribution_gsobol = gsobolDistribution(len(a))
    model_gsobol = ot.PythonFunction(len(a), 1, func_sample=lambda X: gsobol(X, a))
    model_gsobol.setName("G-Sobol")

    # Saltelli
    runConvergence(
        model_gsobol,
        distribution_gsobol,
        SaltelliSensitivityAlgorithm,
        list_sampleSize,
    )

    # Jansen
    runConvergence(
        model_gsobol,
        distribution_gsobol,
        JansenSensitivityAlgorithm,
        list_sampleSize,
    )

    # Mauntz
    runConvergence(
        model_gsobol,
        distribution_gsobol,
        MauntzKucherenkoSensitivityAlgorithm,
        list_sampleSize,
    )

    # Martinez
    runConvergence(
        model_gsobol,
        distribution_gsobol,
        MartinezSensitivityAlgorithm,
        list_sampleSize,
    )

    # ISHIGAMI

    model_ishigami = ishigamiGSymbolic()
    model_ishigami.setName("Ishigami")
    distribution_ishigami = ishigamiDistribution()

    # Saltelli
    runConvergence(
        model_ishigami,
        distribution_ishigami,
        SaltelliSensitivityAlgorithm,
        list_sampleSize,
    )

    # Jansen
    runConvergence(
        model_ishigami,
        distribution_ishigami,
        JansenSensitivityAlgorithm,
        list_sampleSize,
    )

    # Mauntz
    runConvergence(
        model_ishigami,
        distribution_ishigami,
        MauntzKucherenkoSensitivityAlgorithm,
        list_sampleSize,
    )

    # Martinez
    runConvergence(
        model_ishigami,
        distribution_ishigami,
        MartinezSensitivityAlgorithm,
        list_sampleSize,
    )

    # POUTRE

    model_poutre = ot.SymbolicFunction(
        ["L", "b", "h", "E", "F"], ["F * L^3 / (48 * E * b * h^3 / 12)"]
    )
    model_poutre.setName("poutre")
    L = ot.LogNormal()
    L.setParameter(ot.LogNormalMuSigmaOverMu()([5.0, 0.02, 0.0]))
    b = ot.LogNormal()
    b.setParameter(ot.LogNormalMuSigmaOverMu()([0.2, 0.05, 0.0]))
    h = ot.LogNormal()
    h.setParameter(ot.LogNormalMuSigmaOverMu()([0.4, 0.05, 0.0]))
    E = ot.LogNormal()
    E.setParameter(ot.LogNormalMuSigmaOverMu()([3e4, 0.12, 0.0]))
    F = ot.LogNormal()
    F.setParameter(ot.LogNormalMuSigmaOverMu()([0.1, 0.20, 0.0]))
    distribution_poutre = ot.JointDistribution([L, b, h, E, F])

    # Saltelli
    runConvergence(
        model_poutre,
        distribution_poutre,
        SaltelliSensitivityAlgorithm,
        list_sampleSize,
    )

    # Jansen
    runConvergence(
        model_poutre,
        distribution_poutre,
        JansenSensitivityAlgorithm,
        list_sampleSize,
    )

    # Mauntz
    runConvergence(
        model_poutre,
        distribution_poutre,
        MauntzKucherenkoSensitivityAlgorithm,
        list_sampleSize,
    )

    # Martinez
    runConvergence(
        model_poutre,
        distribution_poutre,
        MartinezSensitivityAlgorithm,
        list_sampleSize,
    )



#! /usr/bin/env python

import openturns as ot
import openturns.testing as ott

ot.TESTPREAMBLE()


def test_active_parameter():
    # Define product of matern 1d
    cov_model_1d = ot.MaternModel([0.5], 2.5)
    print("1D Full parameter : ", cov_model_1d.getFullParameter())
    print(
        "1D active cov. param.: ",
        [
            cov_model_1d.getFullParameterDescription()[i]
            for i in cov_model_1d.getActiveParameter()
        ],
    )
    print("Activate nu parameter")
    cov_model_1d.setActiveParameter([0, 2, 3])
    print(
        "active cov. param.: ",
        [
            cov_model_1d.getFullParameterDescription()[i]
            for i in cov_model_1d.getActiveParameter()
        ],
    )

    print("Matern d-dimensional covariance as product")
    d = 3
    cov_model = ot.ProductCovarianceModel([cov_model_1d] * d)
    marginal0 = cov_model.getMarginal(0)
    assert marginal0.getInputDimension() == d, "wrong marginal input dim"
    assert marginal0.getOutputDimension() == 1, "wrong marginal output dim"
    print("Full parameter : ", cov_model.getFullParameter())
    print(
        "active cov. param.: ",
        [
            cov_model.getFullParameterDescription()[i]
            for i in cov_model.getActiveParameter()
        ],
    )
    print("Disable nu for marginals 0 & 1 parameter : ", cov_model.getFullParameter())
    cov_model.setActiveParameter([0, 1, 2, 4, 7])
    print(
        "active cov. param.: ",
        [
            cov_model.getFullParameterDescription()[i]
            for i in cov_model.getActiveParameter()
        ],
    )
    print("Check that active parameter is correctly propagated")

    for k in range(3):
        print(
            "Model ",
            k,
            " : active cov. param.: ",
            [
                cov_model.getCollection()[k].getFullParameterDescription()[i]
                for i in cov_model.getCollection()[k].getActiveParameter()
            ],
        )


def test_active_amplitude_parameter():
    # Define product of matern 1d
    model1 = ot.MaternModel([1.0], 2.5)
    print("Model 1 : ", model1.getFullParameterDescription())
    print("Activate nu parameter and disable sigma2")
    model1.setActiveParameter([0, 1, 3])
    print(
        "model1 active parameter: ",
        [model1.getFullParameterDescription()[i] for i in model1.getActiveParameter()],
    )

    model2 = ot.ExponentiallyDampedCosineModel()
    print("Model 2 : ", model2.getFullParameterDescription())
    print("Activate freq parameter")
    model2.setActiveParameter([0, 2, 3])
    print(
        "model2 active parameter: ",
        [model2.getFullParameterDescription()[i] for i in model2.getActiveParameter()],
    )
    print("Activate nuggetFactor parameter")
    model2.setActiveParameter([0, 1, 2, 3])
    print(
        "model2 active parameter: ",
        [model2.getFullParameterDescription()[i] for i in model2.getActiveParameter()],
    )

    print("Product covariance model")
    cov_model = ot.ProductCovarianceModel([model1, model2])
    print("Full parameter : ", cov_model.getFullParameter())
    print(
        "active cov. param.: ",
        [
            cov_model.getFullParameterDescription()[i]
            for i in cov_model.getActiveParameter()
        ],
    )


def test_parameters_iso():
    scale = []
    nuggetFactor = 1e-12
    amplitude = 1.0
    extraParameter = []

    # model 1
    atom_ex = ot.IsotropicCovarianceModel(ot.MaternModel(), 2)
    atom_ex.setScale([5])
    atom_ex.setAmplitude([1.5])
    scale.append(5)
    amplitude *= 1.5
    extraParameter.append(atom_ex.getKernel().getFullParameter()[-1])

    # model2
    m = ot.MaternModel()
    m.setNu(2.5)
    m.setScale([3])
    m.setAmplitude([3])
    scale.append(3)
    amplitude *= 3
    extraParameter.append(m.getNu())

    # model 3
    atom = ot.IsotropicCovarianceModel(ot.AbsoluteExponential(), 2)
    atom.setScale([2])
    atom.setAmplitude([2.5])
    scale.append(2)
    amplitude *= 2.5

    model = ot.ProductCovarianceModel([atom_ex, m, atom])

    ott.assert_almost_equal(model.getScale(), scale, 1e-16, 1e-16)
    ott.assert_almost_equal(model.getAmplitude(), [amplitude], 1e-16, 1e-16)
    ott.assert_almost_equal(
        model.getFullParameter(),
        scale + [nuggetFactor, amplitude] + extraParameter,
        1e-16,
        1e-16,
    )

    # active parameter should be scale + amplitude
    ott.assert_almost_equal(model.getActiveParameter(), [0, 1, 2, 4], 1e-16, 1e-16)

    # setting new parameters
    extraParameter = [2.5, 0.5]
    model.setFullParameter([6, 7, 8, 0.01, 2] + extraParameter)

    ott.assert_almost_equal(model.getCollection()[0].getScale()[0], 6, 1e-16, 1e-16)
    ott.assert_almost_equal(model.getCollection()[1].getScale()[0], 7, 1e-16, 1e-16)
    ott.assert_almost_equal(model.getCollection()[2].getScale()[0], 8, 1e-16, 1e-16)
    ott.assert_almost_equal(model.getNuggetFactor(), 0.01, 0.0, 0.0)
    ott.assert_almost_equal(model.getAmplitude()[0], 2, 1e-16, 1e-16)
    ott.assert_almost_equal(
        model.getCollection()[0].getFullParameter()[-1], extraParameter[0], 1e-16, 1e-16
    )
    ott.assert_almost_equal(
        model.getCollection()[1].getFullParameter()[-1], extraParameter[1], 1e-16, 1e-16
    )

    # checking active par setting
    model.setActiveParameter([0, 1, 2, 4, 6])
    ott.assert_almost_equal(
        model.getParameter(), [6, 7, 8, 2, extraParameter[-1]], 1e-16, 1e-16
    )


test_active_parameter()
test_active_amplitude_parameter()
test_parameters_iso()



"""
Estimate moments from Taylor expansions
=======================================
In this example we are going to estimate mean and standard deviation of an output variable of interest thanks to the Taylor variance decomposition method of order 1 or 2.
"""

# %%
# Model definition
# ----------------

# %%
import openturns as ot
import openturns.viewer as otv
import numpy as np


# %%
# Create a composite random vector
input_names = ["x1", "x2", "x3", "x4"]
myFunc = ot.SymbolicFunction(input_names, ["cos(x2*x2+x4)/(x1*x1+1+x3^4)"])
R = ot.CorrelationMatrix(4)
for i in range(4):
    R[i, i - 1] = 0.25
distribution = ot.Normal([0.2] * 4, [0.1, 0.2, 0.3, 0.4], R)
distribution.setDescription(input_names)
# We create a distribution-based RandomVector
X = ot.RandomVector(distribution)
# We create a composite RandomVector Y from X and myFunc
Y = ot.CompositeRandomVector(myFunc, X)

# %%
# Taylor expansion based estimation of the moments
# ------------------------------------------------

# %%
# We create a Taylor expansion method to approximate moments
taylor = ot.TaylorExpansionMoments(Y)

# %%
# Analysis of the results
# -----------------------

# %%
# get 1st order mean
print(taylor.getMeanFirstOrder())

# %%
# get 2nd order mean
print(taylor.getMeanSecondOrder())

# %%
# get covariance
print(taylor.getCovariance())

# %%
# draw importance factors
taylor.getImportanceFactors()

# %%
# draw importance factors
graph = taylor.drawImportanceFactors()
view = otv.View(graph)

# %%
# Get the value of the output at the mean point
taylor.getValueAtMean()

# %%
# Get the gradient value of the output at the mean point
taylor.getGradientAtMean()

# %%
# Get the hessian value of the output at the mean point
taylor.getHessianAtMean()

# %%
# Using finite difference gradients
# ---------------------------------

# %%
# When no gradient and/or functions are provided for the model, a finite difference
# approach is relied on automatically. However, it is possible to manually specify
# the characteristic of the considered difference steps.


def myPythonFunction(X):
    x1, x2, x3, x4 = X
    return [np.cos(x2 * x2 + x4) / (x1 * x1 + 1.0 + x3**4)]


myFunc = ot.PythonFunction(4, 1, myPythonFunction)

# %%
# For instance, a user-defined constant step value can be considered
gradEpsilon = [1e-8] * 4
hessianEpsilon = [1e-7] * 4
gradStep = ot.ConstantStep(gradEpsilon)  # Costant gradient step
hessianStep = ot.ConstantStep(hessianEpsilon)  # Constant Hessian step
myFunc.setGradient(
    ot.CenteredFiniteDifferenceGradient(gradStep, myFunc.getEvaluation())
)
myFunc.setHessian(
    ot.CenteredFiniteDifferenceHessian(hessianStep, myFunc.getEvaluation())
)

# %%
# Alternatively, we can consider a finite difference step value which
# depends on the location in the input space by relying on the BlendedStep class:
gradEpsilon = [1e-8] * 4
hessianEpsilon = [1e-7] * 4
gradStep = ot.BlendedStep(gradEpsilon)  # Costant gradient step
hessianStep = ot.BlendedStep(hessianEpsilon)  # Constant Hessian step
myFunc.setGradient(
    ot.CenteredFiniteDifferenceGradient(gradStep, myFunc.getEvaluation())
)
myFunc.setHessian(
    ot.CenteredFiniteDifferenceHessian(hessianStep, myFunc.getEvaluation())
)

# %%
# We can then proceed in the same way as before
Y = ot.CompositeRandomVector(myFunc, X)
taylor = ot.TaylorExpansionMoments(Y)

# %%
otv.View.ShowAll()



#! /usr/bin/env python

import openturns as ot
import openturns.testing as ott

ot.TESTPREAMBLE()

# Data & estimation
inputDimension = 1
X = ot.Normal().getSample(100)
X = X.sortAccordingToAComponent(0)
covarianceModel = ot.SquaredExponential([1.0], [1.0])
model = ot.SymbolicFunction(["x"], ["x - 0.6 * cos(x/3)"])
Y = model(X)
basis = ot.QuadraticBasisFactory(inputDimension).build()
algo = ot.GaussianProcessFitter(X, Y, covarianceModel, basis)
algo.setKeepCholeskyFactor(False)
algo.setOptimizationAlgorithm(ot.NLopt("LN_NELDERMEAD"))
algo.run()

# perform an evaluation
result = algo.getResult()
metaModel = result.getMetaModel()
conditionalCovariance = result.getCovarianceModel()
residual = metaModel(X) - Y
ott.assert_almost_equal(residual.computeCentralMoment(2), [1.06e-05], 1e-5, 1e-5)
print(conditionalCovariance.getParameter())
ott.assert_almost_equal(
    conditionalCovariance.getParameter(), [0.620868, 0.000941836], 5e-3, 1e-3
)
likelihood = algo.getReducedLogLikelihoodFunction()
assert likelihood.getInputDimension() == 1, "likelihood dim"



"""
Bandwidth sensitivity in kernel smoothing
=========================================
"""

# %%
#
# Introduction
# ------------
#
# When we have a sample, we may estimate the probability density function of the
# underlying distribution using kernel smoothing.
# One of the parameters of this method is the bandwidth, which can be either
# set by the user, or estimated from the data.
# This is especially true when the density is multimodal.
# In this example, we consider a bimodal distribution and see how the bandwidth
# can change the estimated probability density function.
#
# We consider the distribution:
#
# .. math::
#    f_1(x) = w_1 f_A(x) + w_2 f_B(x)
#
#
# for any :math:`x\in\mathbb{R}` where :math:`f_A` is the density of the Normal distribution
# :math:`\mathcal{N}(0,1)`, :math:`f_B` is the density of the Normal distribution
# :math:`\mathcal{N}(3/2,(1/3)^2)` and the weights are :math:`w_1 = \frac{3}{4}` and :math:`w_2 = \frac{1}{4}`.
#
# This is a mixture of two Normal distributions: 1/4th of the observations have the :math:`\mathcal{N}(0,1)`
# distribution and 3/4th of the observations have the :math:`\mathcal{N}(3/2,(1/3)^2)` distribution.
# This example is considered in (Wand, Jones, 1994), page 14, Figure 2.3.
#
# We consider a sample generated from independent realizations of :math:`f_1` and
# want to approximate the distribution from kernel smoothing.
# More precisely, we want to observe the sensitivity of the resulting density to the
# bandwidth.
#
# References
# ----------
#
# * "Kernel Smoothing", M.P.Wand, M.C.Jones. Chapman and Hall / CRC (1994).
#

# %%
# Generate the mixture by merging two samples
# -------------------------------------------
#
# In this section, we show that a mixture of two Normal distributions is
# nothing more than the merged sample of two independent Normal distributions.
# In order to generate a sample with size :math:`n`, we sample
# :math:`\lfloor w_1 n\rfloor` points from the first Normal distribution
# :math:`f_A` and :math:`\lfloor w_2 n\rfloor` points from the second Normal
# distribution :math:`f_B`. Then we merge the two samples.

# %%
import openturns as ot
import openturns.viewer as otv

# %%
# We choose a rather large sample size: :math:`n=1000`.

# %%
n = 1000

# %%
# Then we define the two Normal distributions and their parameters.

# %%
w1 = 0.75
w2 = 1.0 - w1
distribution1 = ot.Normal(0.0, 1.0)
distribution2 = ot.Normal(1.5, 1.0 / 3.0)

# %%
# We generate two independent sub-samples from the two Normal distributions.

# %%
sample1 = distribution1.getSample(int(w1 * n))
sample2 = distribution2.getSample(int(w2 * n))

# %%
# Then we merge the sub-samples into a larger one with the `add` method of the :class:`~openturns.Sample` class.

# %%
sample = ot.Sample(sample1)
sample.add(sample2)
sample.getSize()

# %%
# In order to see the result, we build a kernel smoothing approximation on the sample. In order to keep it simple, let us use the default bandwidth selection rule.

# %%
factory = ot.KernelSmoothing()
fit = factory.build(sample)

# %%
graph = fit.drawPDF()
view = otv.View(graph)

# %%
# We see that the distribution of the merged sample has two modes.
# However, these modes are not clearly distinct. To distinguish them, we could increase the sample size.
# However, it might be interesting to see if the bandwidth selection rule can be better chosen: this is the purpose of the next section.

# %%
# Simulation based on a mixture
# -----------------------------
#
# Since the distribution that we approximate is a mixture, it will be more convenient to create it from the :class:`~openturns.Mixture` class.
# It takes as input argument a list of distributions and a list of weights.

# %%
distribution = ot.Mixture([distribution1, distribution2], [w1, w2])

# %%
# Then we generate a sample from it.

# %%
sample = distribution.getSample(n)

# %%
factory = ot.KernelSmoothing()
fit = factory.build(sample)

# %%
factory.getBandwidth()

# %%
# We see that the default bandwidth is close to 0.17.

# %%
graph = distribution.drawPDF()
curve = fit.drawPDF()
graph.add(curve)
graph.setLegends(["Mixture", "Kernel smoothing"])
graph.setLegendPosition("upper left")
view = otv.View(graph)

# %%
# We see that the result of the kernel smoothing approximation of the mixture is similar to the previous one and could be improved.

# %%
# Sensitivity to the bandwidth
# ----------------------------

# %%
# In this section, we observe the sensitivity of the kernel smoothing to the bandwidth.
# We consider the three following bandwidths: the small bandwidth 0.05, the large bandwidth 0.54 and 0.18 which is in-between.
# For each bandwidth, we use the second optional argument of the `build` method in order to select a specific bandwidth value.

# %%
hArray = [0.05, 0.54, 0.18]
nLen = len(hArray)
grid = ot.GridLayout(1, len(hArray))
index = 0
for i in range(nLen):
    fit = factory.build(sample, [hArray[i]])
    graph = fit.drawPDF()
    exact = distribution.drawPDF()
    curve = exact.getDrawable(0)
    curve.setLegend("Mixture")
    curve.setLineStyle("dashed")
    graph.add(curve)
    graph.setXTitle("X")
    graph.setTitle("h=%.4f" % (hArray[i]))
    graph.setLegends([""])
    bounding_box = graph.getBoundingBox()
    upper_bound = bounding_box.getUpperBound()
    upper_bound[1] = 0.5  # Common y-range
    graph.setBoundingBox(bounding_box)
    grid.setGraph(0, index, graph)
    index += 1

view = otv.View(grid, figure_kw={"figsize": (10.0, 4.0)})
# %%
# We see that when the bandwidth is too small, the resulting kernel smoothing has many more modes than the distribution it is supposed to approximate.
# When the bandwidth is too large, the approximated distribution is too smooth and has only one mode instead of the expected two modes which are in the mixture distribution.
# When the bandwidth is equal to 0.18, the two modes are correctly represented.

# %%
# Sensitivity to the bandwidth rule
# ---------------------------------
#
# The library provides three different rules to compute the bandwidth. In this section, we compare the results that we can get with them.

# %%
h1 = factory.computeSilvermanBandwidth(sample)[0]
h1

# %%
h2 = factory.computePluginBandwidth(sample)[0]
h2

# %%
h3 = factory.computeMixedBandwidth(sample)[0]
h3

# %%
factory.getBandwidth()[0]

# %%
# We see that the default rule is the "Mixed" rule. This is because the sample is in dimension 1 and the sample size is quite large.
# For a small sample in 1 dimension, the "Plugin" rule would have been used.
#
# The following script compares the results produced by the three rules.

# %%
hArray = [h1, h2, h3]
legends = ["Silverman", "Plugin", "Mixed"]
nLen = len(hArray)
grid = ot.GridLayout(1, len(hArray))
index = 0
for i in range(nLen):
    fit = factory.build(sample, [hArray[i]])
    graph = fit.drawPDF()
    exact = distribution.drawPDF()
    curve = exact.getDrawable(0)
    curve.setLegend("Mixture")
    curve.setLineStyle("dashed")
    graph.add(curve)
    graph.setLegends([""])
    graph.setTitle("h=%.4f, %s" % (hArray[i], legends[i]))
    graph.setXTitle("X")
    if i > 0:
        graph.setYTitle("")
    bounding_box = graph.getBoundingBox()
    upper_bound = bounding_box.getUpperBound()
    upper_bound[1] = 0.5  # Common y-range
    graph.setBoundingBox(bounding_box)
    grid.setGraph(0, index, graph)
    index += 1

view = otv.View(grid, figure_kw={"figsize": (10.0, 4.0)})

otv.View.ShowAll()
# %%
# We see that the bandwidth produced by Silverman's rule is too large, leading to an oversmoothed distribution.
# The results produced by the Plugin and Mixed rules are comparable in this case.



"""
FAST sensitivity indices
========================
"""

# %%
# This example will demonstrate how to quantify the correlation between the input
# variables and the output variable of a model using the FAST method,
# based upon the Fourier decomposition of the model response,
# which is a relevant alternative to the classical
# simulation approach for computing Sobol sensitivity indices.
#
# The FAST indices, like the Sobol indices, allow one to
# evaluate the importance of a single variable or a specific set of variables.
#
# In theory, FAST indices range is :math:`\left[0; 1\right]` ; the closer to 1 the
# index is, the greater the model response sensitivity to the variable is.
#
# The FAST method compute the first and total order indices.
# The first order indices evaluate the importance of one variable at a time
# (:math:`d` indices, with :math:`d` the input dimension of the model).
#
# The :math:`d` total indices give the relative importance of every variables except
# the variable :math:`X_i`, for every variable.

# %%
from openturns.usecases import ishigami_function
import openturns as ot
import openturns.viewer as otv

# %%
# We load the :ref:`Ishigami model <use-case-ishigami>` from the usecases module :
im = ishigami_function.IshigamiModel()

# %%
# The `IshigamiModel` data class contains the input independent joint distribution :
distribution = im.distribution

# %%
# and the Ishigami function :
model = im.model


# %%
size = 400
sensitivityAnalysis = ot.FAST(model, distribution, size)
# Compute the first order indices (first and total order indices are
# computed together)
firstOrderIndices = sensitivityAnalysis.getFirstOrderIndices()
# Retrieve total order indices
totalOrderIndices = sensitivityAnalysis.getTotalOrderIndices()

# %%
# Print indices
print("First order FAST indices:", firstOrderIndices)
print("Total order FAST indices:", totalOrderIndices)

# %%
graph = ot.SobolIndicesAlgorithm.DrawImportanceFactors(
    firstOrderIndices, distribution.getDescription(), "FAST first order indices"
)
view = otv.View(graph)

# %%
graph = ot.SobolIndicesAlgorithm.DrawImportanceFactors(
    totalOrderIndices, distribution.getDescription(), "FAST total order indices"
)
view = otv.View(graph)

# %%
# Display all figures
otv.View.ShowAll()



"""
Taylor approximations
=====================
"""

# %%
# In this example we build a local approximation of a model using the
# Taylor decomposition with the :class:`~openturns.LinearTaylor` class.
#
# We consider the function :math:`\vect{h} : \Rset^2 \rightarrow \Rset^2` defined by:
#
# .. math::
#    \vect{h}(\vect{x}) = \Tr{\left( \cos(x_1 + x_2), (x_2 + 1) e^{x_1 - 2 x_2} \right)}.
#
# for any :math:`\vect{x} \in \Rset^2`.
# The metamodel :math:`\widehat{\vect{h}}` is an approximation of the model :math:`\vect{h}`:
#
# .. math::
#    \vect{y} \approx \widehat{\vect{h}}(\vect{x})
#
# for any :math:`\vect{x} \in \Rset^2`. We note  :math:`h_k`, :math:`k=1, 2` the marginal outputs
# In this example, we consider two different approximations:
#
# - the first order Taylor expansion,
# - the second order Taylor expansion.

# %%
import openturns as ot
import openturns.viewer as otv


# %%
# Define the model
# ~~~~~~~~~~~~~~~~

# %%
# Prepare some data.
formulas = ["cos(x1 + x2)", "(x2 + 1) * exp(x1 - 2 * x2)"]
model = ot.SymbolicFunction(["x1", "x2"], formulas)

# %%
# Center of the approximation.
x0 = [-0.4, -0.4]

# %%
# Drawing bounds.
a = -0.4
b = 0.0

# %%
# First order Taylor expansion
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# %%
# Let :math:`\vect{x}_0 \in \Rset^2` be a reference point where
# the linear approximation is evaluated.
# The first order Taylor expansion of the output marginal :math:`h_k` is:
#
# .. math::
#    \widehat{h_k}(\vect{x}) = h_k(\vect{x}_0) +
#      \sum_{i=1}^{2} \frac{\partial h_k}{\partial x_i}(\vect{x}_0) \left(x_i - x_{0,i} \right)
#
# for any :math:`\vect{x} \in \Rset^2`.


# %%
# Create a linear (first-order) Taylor approximation.
algo = ot.LinearTaylor(x0, model)
algo.run()
responseSurface = algo.getMetaModel()

# %%
# Plot the second output of our model with :math:`x_1=x_{0,1}`.

# %%
graph = ot.ParametricFunction(model, [0], [x0[1]]).getMarginal(1).draw(a, b)
graph.setLegends(["Model"])
curve = (
    ot.ParametricFunction(responseSurface, [0], [x0[1]])
    .getMarginal(1)
    .draw(a, b)
    .getDrawable(0)
)
curve.setLegend("Taylor")
curve.setLineStyle("dashed")
graph.add(curve)
graph.setLegendPosition("upper right")
view = otv.View(graph)

# %%
# Second order Taylor expansion
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# %%
# Let :math:`\vect{x}_0 \in \Rset^2` be a reference point where
# the quadratic approximation is evaluated.
# The second order Taylor expansion of the output marginal :math:`h_k` is:
#
# .. math::
#
#     \widehat{h_k}(\vect{x}) = h_k(\vect{x}_0) + \sum_{i=1}^{2}
#     \frac{\partial h_k}{\partial x_i}(\vect{x}_0) \left(x_i - x_{0,i} \right) +
#     \frac{1}{2} \sum_{i,j=1}^{2}
#     \frac{\partial^2 h_k}{\partial x_i \partial x_j}(\vect{x}_0) \left(x_i - x_{0,i} \right) \left(x_j - x_{0,j} \right)
#
# for any :math:`\vect{x} \in \Rset^2`.

# %%
# Create a quadratic (second-order) Taylor approximation.

# %%
algo = ot.QuadraticTaylor(x0, model)
algo.run()
responseSurface = algo.getMetaModel()

# %%
# Plot second output of our model with :math:`x_1=x_{0,1}`.

# %%
graph = ot.ParametricFunction(model, [0], [x0[1]]).getMarginal(1).draw(a, b)
graph.setLegends(["Model"])
curve = (
    ot.ParametricFunction(responseSurface, [0], [x0[1]])
    .getMarginal(1)
    .draw(a, b)
    .getDrawable(0)
)
curve.setLegend("Taylor")
curve.setLineStyle("dashed")
graph.add(curve)
graph.setLegendPosition("upper right")
view = otv.View(graph)

# %%
view.ShowAll()



"""
Use the Ratio of Uniforms algorithm to sample a distribution
============================================================
"""

# %%
# In this example, we illustrate in dimension :math:`\inputDim = 1` the ratio of uniforms algorithm  to sample a distribution which
# probability density function is known up to a normalization factor.
#
# Consider a scalar distribution whose
# probability density function :math:`p` is known up to a normalization factor:
# let :math:`f: \Rset \rightarrow \Rset^+` be a function such that
# :math:`p(x) = cf(x)` for all :math:`x \in \Rset` with :math:`c \in \Rset^+_*`.
#
# Let :math:`r \in \Rset^+_*`. Let  :math:`A_{f,r}` be the domain defined by:
#
# .. math::
#
#    A_{f,r} = \left\{ (u, v) \in \Rset^2 \, |\, 0 \leq u \leq f\left(\dfrac{v}{u^r}\right)^{\frac{1}{1+r}}\right \}
#
# The Lebesgue measure of :math:`A_{f,r}`  is equal to :math:`\dfrac{1}{c(1+r)}`.
#
# Let :math:`(U, V)` be a random variable uniformly distributed on :math:`A_{f,r}`.
# Then, :math:`X = \dfrac{V}{U^r}` is a random variable on
# :math:`\Rset` distributed according to :math:`p`.
#
# Under the condition that :math:`f(x)^{\frac{1}{1+r}}` and
# :math:`x f(x)^{\frac{1}{1+r}}` are bounded, then the domain
# :math:`A_{f,r}` is bounded and can be included in a rectangular bounding box :math:`\tilde{A}_{f,r}`
# defined by:
#
# .. math::
#
#    \tilde{A}_{f,r} & = \left[0, \sup_{x} f(x)^{\frac{1}{1+r}} \right] \times
#    \left[ \inf_{x} x f(x)^{\frac{1}{1+r}} ,  \sup_{x} x f(x)^{\frac{1}{1+r}}\right]\\
#                    & = \left[0, u_{sup} \right] \times \left[v_{inf}, v_{sup}\right]
#
# Note that :math:`u_{sup} > 0`, :math:`v_{sup} \geq 0` and  :math:`v_{inf} \leq 0`.
#
# This allows one to sample uniformly the domain :math:`A_{f,r}` by rejection sampling inside
# :math:`\tilde{A}_{f,r}`.
#
# Study of the :math:`(u,v)`-space.
# ---------------------------------
# The boundary of :math:`\tilde{A}_{f,r}` is defined by:
#
# .. math::
#
#     \partial \tilde{A}_{f,r} = \cF_1 \cup \cF_2 \cup \cF_3 \cup \cF_4
#
# where the boundaries :math:`\cF_i` are defined by:
#
# .. math::
#
#    \cF_1 & = \left \{ (u, v_{sup}), 0 \leq u \leq u_{sup} \right \} \\
#    \cF_2 & = \left \{ (u, v_{inf}), 0 \leq u \leq u_{sup} \right \}\\
#    \cF_3 & = \left \{ (u_{sup}, v), v_{inf} \leq v \leq v_{sup} \right \}\\
#    \cF_4 & = \left \{ (0, v), v_{inf} \leq v \leq v_{sup} \right \}
#
# The boundary of :math:`A_{f,r}` is defined by:
#
# .. math::
#
#    \partial A_{f,r}  = \left\{ (u,v) = \left( f(x)^{\dfrac{1}{1+r}}, x f(x)^{\dfrac{r}{1+r}} \right),
#    x \in \Rset \right\}
#
# The following function draws the domain :math:`A_{f,r}` and the bounding box :math:`\partial \tilde{A}_{f,r}`
# in the :math:`(u,v)`-space.
import openturns as ot
import openturns.viewer as otv
import openturns.experimental as otexp
from math import pi, exp


def draw_RatioOfUniforms_UV(ratio_algo):
    r = ratio_algo.getR()
    range_logF = ratio_algo.getRange()
    xInf = range_logF.getLowerBound()[0]
    xSup = range_logF.getUpperBound()[0]
    uSup = ratio_algo.getSupU()
    vInf = ratio_algo.getInfV()[0]
    vSup = ratio_algo.getSupV()[0]
    logF = ratio_algo.getLogUnscaledPDF()

    # \partial A_{f,r} domain
    x = xInf
    u = 0.0
    v = 0.0
    xPrec = None
    uPrec = None
    vPrec = None
    first = True
    step = 0.01 * (uSup + vSup - vInf)
    delta = step
    data_UV = ot.Sample(0, 2)
    while x < xSup:
        val_F = exp(logF([x])[0])
        u = val_F ** (1.0 / (1.0 + r))
        v = x * val_F ** (r / (1.0 + r))
        # Step adaptation after the first point
        if not first:
            # If the step is too large
            while abs(u - uPrec) + abs(v - vPrec) > 2 * delta:
                step *= 0.5
                x = xPrec + step
                if x < xSup:
                    val_F = exp(logF([x])[0])
                    u = val_F ** (1.0 / (1.0 + r))
                    v = x * val_F ** (r / (1.0 + r))
            # If the step is too small
            while abs(u - uPrec) + abs(v - vPrec) < 0.5 * delta and x < xSup:
                step *= 2.0
                x = xPrec + step
                if x < xSup:
                    val_F = exp(logF([x])[0])
                    u = val_F ** (1.0 / (1.0 + r))
                    v = x * val_F ** (r / (1.0 + r))
        xPrec = x
        uPrec = u
        vPrec = v
        x += step
        first = False
        data_UV.add([u, v])
    # to get a closed boundary
    data_UV.add(data_UV[0])
    g = ot.Graph("", "u", "v")
    g.setLegendPosition("bottomright")
    data_A = ot.Curve(data_UV)
    data_A.setColor("purple")
    g.add(data_A)

    # \partial \tilde{A}_{f,r} bounding box
    boundary_1 = ot.Curve([[0, vSup], [uSup, vSup]])
    boundary_1.setColor("blue")
    boundary_2 = ot.Curve([[0, vInf], [uSup, vInf]])
    boundary_2.setColor("red")
    boundary_3 = ot.Curve([[uSup, vInf], [uSup, vSup]])
    boundary_3.setColor("green")
    boundary_4 = ot.Curve([[0, vInf], [0, vSup]])
    boundary_4.setColor("black")
    g.add(boundary_1)
    g.add(boundary_2)
    g.add(boundary_3)
    g.add(boundary_4)

    leg = g.getLegends()
    leg[0] = r"boundary of $A_{f,r}$"
    leg[1] = r"$\mathcal{F}_1$"
    leg[2] = r"$\mathcal{F}_2$"
    leg[3] = r"$\mathcal{F}_3$"
    leg[4] = r"$\mathcal{F}_4$"
    g.setLegends(leg)

    return g


# %%
# Study of the :math:`(x,y)`-space.
# ---------------------------------
# We use the mapping function :math:`\varphi` defined by:
#
# .. math::
#
#    \varphi: & \Rset^2 \rightarrow \Rset^2 \\
#             & (u,v)  \rightarrow \left( \dfrac{v}{u^r}, u \right)
#
# Then we have:
#
# .. math::
#
#    \varphi(A_{f,r}) = \left\{ (x,y) \, |\, x \in \Rset, 0 \leq y \leq f(x)^{\dfrac{1}{1+r}} \right\}
#
# and:
#
# .. math::
#
#     \partial \varphi(\tilde{A}_{f,r}) = \varphi(\cF_1) \cup \varphi(\cF_2) \cup \varphi(\cF_3)
#
# where the boundaries :math:`\varphi(\cF_i)` are defined by:
#
# .. math::
#
#    \varphi(\cF_1) & = \left\{ \varphi(u, v_{sup}), 0 \leq u \leq u_{sup} \right\}\\
#          & = \left\{ \left(x,\left(\dfrac{v_{sup}}{x}\right)^{1/r} \right),
#              x \geq \dfrac{v_{sup}}{u_{sup}^{r}} \right \}
#
# .. math::
#
#    \varphi(\cF_2) & = \left \{ \varphi(u, v_{inf}), 0 \leq u \leq u_{sup} \right\}\\
#          & = \left\{ \left(x,\left(\dfrac{v_{inf}}{x}\right)^{1/r}\right),
#              x \leq \dfrac{v_{inf}}{u_{sup}^{r}} \right \}
#
# .. math::
#
#    \varphi(\cF_3) & = \left\{ \varphi(u_{sup}, v), v_{inf} \leq v \leq v_{sup} \right\}\\
#          & = \left\{ (x, u_{sup}),
#              \dfrac{v_{inf}}{u_{sup}^{r}} \leq x \leq \dfrac{v_{sup}}{u_{sup}^{r}}  \right\}
#
# The following function draws the boundary :math:`\partial \varphi(A_{f,r})` and the bounding box
# :math:`\partial \tilde{A}_{f,r}` in the :math:`(x,y)`-space.


def draw_RatioOfUniforms_XY(ratio_algo, nPoints):
    r = ratio_algo.getR()
    range_logF = ratio_algo.getRange()
    xInf = range_logF.getLowerBound()[0]
    xSup = range_logF.getUpperBound()[0]
    uSup = ratio_algo.getSupU()
    vInf = ratio_algo.getInfV()[0]
    vSup = ratio_algo.getSupV()[0]
    logF = ratio_algo.getLogUnscaledPDF()

    # \partial varpĥi(A_{f,r}) domain
    eps = 1e-10
    step = (xSup - xInf - 2.0 * eps) / (nPoints - 1)
    data_XY = ot.Sample(nPoints, 2)
    for i in range(nPoints):
        x = xInf + eps + i * step
        val_F = exp(logF([x])[0])
        data_XY[i, 0] = x
        data_XY[i, 1] = val_F ** (1.0 / (1.0 + r))
    g = ot.Graph("", "x", "y")
    g.setLegendPosition("lower center")
    data_A = ot.Curve(data_XY)
    data_A.setColor("purple")
    g.add(data_A)

    # \partial varphi(\tilde{A}_{f,r}) bounding box
    leg = ot.Description()
    leg.add(r"boundary of $\varphi(A_{f,r})$")
    boundary_1_fct = ot.SymbolicFunction(
        "x", "(" + str(vSup) + "/x)^(1/" + str(r) + ")"
    )
    if vSup / (uSup**r) < xSup:
        boundary_1 = boundary_1_fct.draw(vSup / (uSup**r), xSup, 101)
        boundary_1.setColors(["blue"])
        leg.add(r"$\varphi(\mathcal{F}_1)$")
        g.add(boundary_1)
    if vInf / (uSup**r) < uSup:
        boundary_2_fct = ot.SymbolicFunction(
            "x", "(" + str(vInf) + "/x)^(1/" + str(r) + ")"
        )
        boundary_2 = boundary_2_fct.draw(xInf, vInf / (uSup**r), 101)
        boundary_2.setColors(["red"])
        leg.add(r"$\varphi(\mathcal{F}_2)$")
        g.add(boundary_2)
    boundary_3 = ot.Curve([[vInf / (uSup**r), uSup], [vSup / (uSup**r), uSup]])
    boundary_3.setColor("green")
    g.add(boundary_3)
    leg.add(r"$\varphi(\mathcal{F}_3)$")
    g.setLegends(leg)

    return g


# %%
# Illustration.
# -------------
# We create the function :math:`\log f` defined by:
#
# .. math::
#    :label: robertEx
#
#    \log f(x) = \log(\cos(x)) + x,  \quad  x \in \left[-\dfrac{\pi}{2}, \dfrac{\pi}{2}\right]
#
# and the :class:`~openturns.experimental.RatioOfUniforms`.


log_UnscaledPDF = ot.SymbolicFunction("x", "log(cos(x)) + x")
eps = 1e-5
range_PDF = ot.Interval(-pi / 2.0 + eps, pi / 2.0 - eps)
ratio_Algo = otexp.RatioOfUniforms(log_UnscaledPDF, range_PDF)

# %%
# We draw the boundary :math:`\partial A_{f,r}` and the bounding box :math:`\partial \tilde{A}_{f,r}`
# in the :math:`(u,v)`-space.
g_UV = draw_RatioOfUniforms_UV(ratio_Algo)
g_UV.setTitle(
    rf"$r = ${ratio_Algo.getR():.3g}, $\tau = ${ratio_Algo.getAcceptanceRatio():.3g}"
)

# %%
# We draw the boundary :math:`\partial \varphi(A_{f,r})` and the bounding box :math:`\partial \varphi(\tilde{A}_{f,r})`
# in the :math:`(x,y)`-space.
# sphinx_gallery_thumbnail_number = 1
g_XY = draw_RatioOfUniforms_XY(ratio_Algo, 1001)
g_XY.setTitle(
    rf"$r = ${ratio_Algo.getR():.3g}, $\tau = ${ratio_Algo.getAcceptanceRatio():.3g}"
)

grid_default = ot.GridLayout(1, 2)
grid_default.setGraph(0, 0, g_UV)
grid_default.setGraph(0, 1, g_XY)
grid_default.setTitle(r"$\log f(x) = \log(\cos(x)) + x, x \in ]-\pi/2, \pi/2[$")
view = otv.View(grid_default)

# %%
# We change the :math:`r` value and we draw the same plots.
grid_full = ot.GridLayout(4, 2)
for i, r in enumerate([0.5, 0.8, 1.2, 2.0]):
    ratio_Algo.setR(r)
    g_UV = draw_RatioOfUniforms_UV(ratio_Algo)
    g_UV.setLegendPosition("")
    g_UV.setTitle(
        rf"$r = ${ratio_Algo.getR():.3g}, $\tau = ${ratio_Algo.getAcceptanceRatio():.3g}"
    )

    g_XY = draw_RatioOfUniforms_XY(ratio_Algo, 1001)
    g_XY.setLegendPosition("")
    g_XY.setTitle(
        rf"$r = ${ratio_Algo.getR():.3g}, $\tau = ${ratio_Algo.getAcceptanceRatio():.3g}"
    )

    grid_full.setGraph(i, 0, g_UV)
    grid_full.setGraph(i, 1, g_XY)

grid_full.setTitle(r"$\log f(x) = \log(\cos(x)) + x,  \quad x \in ]-\pi/2, \pi/2[$")
view = otv.View(grid_full)

# %%
# Study of the acceptance ratio.
# ------------------------------
# We study the acceptance ratio :math:`\tau(r)` as a function of the parameter :math:`r` for some distributions.
# In these cases, the value is exact and computed from the volume of :math:`A_{f,r}` and the one of :math:`\tilde{A}_{f,r}`.
# We draw the function :math:`r \rightarrow \tau(r)`. We conclude that the default value :math:`r=1` seems to be correct
# in all these cases.
#
mixt_gaus = ot.Mixture([ot.Normal(-2.0, 1.0), ot.Normal(4.0, 1.0)], [0.2, 0.8])
logPDF_list = [
    ot.Normal().getLogPDF(),
    ot.Normal(2).getLogPDF(),
    ot.Normal(3).getLogPDF(),
    ot.Exponential().getLogPDF(),
    mixt_gaus.getLogPDF(),
]
range_list = [
    ot.Normal().getRange(),
    ot.Normal(2).getRange(),
    ot.Normal(3).getRange(),
    ot.Exponential().getRange(),
    mixt_gaus.getRange(),
]
name_list = ["Normal(1)", "Normal(2)", "Normal(3)", "Exponential()", "Mixture Normal"]

g = ot.Graph(r"Acceptance ratio $\tau$ as a function of $r$", r"$r$", r"$\tau$")
g.setLegendPosition("upper right")
g.setLogScale(1)
numberR = 50
a = (1e2) ** (2.0 / numberR)
data_r_ratio = ot.Sample(numberR + 1, 2)
isScaled = True
for k in range(len(logPDF_list)):
    ratio_Algo = otexp.RatioOfUniforms(logPDF_list[k], range_list[k], isScaled)
    for i in range(numberR + 1):
        r = a ** (i - 0.5 * numberR)
        ratio_Algo.setR(r)
        accept_Ratio = ratio_Algo.getAcceptanceRatio()
        data_r_ratio[i] = [r, accept_Ratio]
    draw = ot.Curve(data_r_ratio)
    draw.setLegend(name_list[k])
    g.add(draw)

isScaled = False
ratio_Algo = otexp.RatioOfUniforms(log_UnscaledPDF, range_PDF, isScaled)
for i in range(numberR + 1):
    r = a ** (i - 0.5 * numberR)
    ratio_Algo.setR(r)
    accept_Ratio = ratio_Algo.getAcceptanceRatio()
    data_r_ratio[i] = [r, accept_Ratio]
draw = ot.Curve(data_r_ratio)
draw.setLegend(
    r"$f(x) = \cos(x)e^x\mathbf{1}_{\left[-\frac{\pi}{2}, \frac{\pi}{2}\right]}(x)$"
)
g.add(draw)

g.setLegendPosition("upper left")
g.setLegendCorner([1.0, 1.0])
view = otv.View(g)

# %%
# Exploration of the optimization problems.
# -----------------------------------------
# We illustrate how to explore the results of the optimization problems that have been solved to define
# :math:`\tilde{A}_{f,r}`.
# We consider the example defined by the function:
#
# .. math::
#
#     f(x) = \frac{1}{2} (2 + \sin(x)^2) \exp \left[- \left(2 + \cos(3x)^3 + \sin(2x)^3 \right) x
#     \right]  \mathbf{1}_{[0, 2 \pi]}(x).
#
# The method *initialize* returns all the starting points that have been used to compute respectively :math:`u_{sup}`,
# :math:`v_{inf}` and :math:`v_{sup}`.
# The maximum number of starting points is defined in :class:`~openturns.ResourceMap`, entry
# *RatioOfUniformas-MaximumMultiStart*. These
# points are selected scrolling through  the Sobol low discrepance sequence in dimension :math:`\inputDim =  1`
# and which maximum size is defined in :class:`~openturns.ResourceMap`, entry *RatioOfUniforms-CandidateNumber*.
f = ot.SymbolicFunction(
    "x", "0.5 * (2 + sin(x)^2) * exp( -( 2 + cos(3*x)^3 + sin(2*x)^3) * x )"
)
log_UnscaledPDF = ot.ComposedFunction(ot.SymbolicFunction("x", "log(x)"), f)
lower_bound = 0.0
upper_bound = 2.0 * pi
range_PDF = ot.Interval(lower_bound, upper_bound)

graph = f.draw(lower_bound, upper_bound, 512)
graph.setTitle(
    r"Christian Robert function: $f(x) =  0.5(2 + \sin^2 x) e^{ -x( 2 + \cos^3 3x + \sin^3 2x)}, x \in [0, 2\pi]$"
)
graph.setXTitle(r"$x$")
graph.setYTitle(r"$f(x)$")
view = otv.View(graph)

ratio_Algo = otexp.RatioOfUniforms(log_UnscaledPDF, range_PDF)

# %%
# In order to get all the intermediate results, we change the :class:`~openturns.ResourceMap`,
# entry *MultiStart-KeepResults*.
#
# Furthermore, as :math:`I = [0, 2 \pi]=  [a_i, b_i]`, then :math:`a_i \geq 0` implies that :math:`v_{inf} = 0`.
# There are only the bounds :math:`u_{sup}` and :math:`v_{sup}` to find: the collection of optimization problems
# only contains 2 items.
ot.ResourceMap.SetAsBool("MultiStart-KeepResults", True)
coll_Multi_Start = ratio_Algo.initialize()
start_Sample_SupU = coll_Multi_Start[0].getStartingSample()
start_Sample_SupV = coll_Multi_Start[1].getStartingSample()

# %%
# We plot on the previous graph all the starting points (red) used to find :math:`u_{sup}` and the associated maxima found by the algorithm. We
# also link each starting point to its associated optimal value (green).
Cloud_start_Usup = ot.Cloud(
    start_Sample_SupU, ot.Sample(start_Sample_SupU.getSize(), [0.0])
)
Cloud_start_Usup.setColor("red")
Cloud_start_Usup.setPointStyle("fsquare")
graph.add(Cloud_start_Usup)

coll_results = coll_Multi_Start[0].getResultCollection()
sample_optima = ot.Sample(0, 2)
for i in range(coll_results.getSize()):
    opt_x = coll_results[i].getOptimalPoint()[0]
    opt_value = f([opt_x])[0]
    sample_optima.add([opt_x, opt_value])
    line_convergence = ot.Curve([[start_Sample_SupU[i, 0], 0.0], [opt_x, opt_value]])
    line_convergence.setLineStyle("dashed")
    line_convergence.setColor("black")
    graph.add(line_convergence)
cloud_optima = ot.Cloud(sample_optima)
cloud_optima.setColor("green")
cloud_optima.setPointStyle("star")
graph.add(cloud_optima)
graph.setYTitle("")
view = otv.View(graph)


# %%
view.ShowAll()



#! /usr/bin/env python

import openturns as ot
import openturns.testing as ott

ot.TESTPREAMBLE()
ot.RandomGenerator.SetSeed(0)

# We create a Sample
point1 = [10.0, 20.0]
point2 = [11.0, 21.0]
point3 = [12.0, 22.0]
point4 = [15.0, 25.0]
sample1 = ot.Sample([point1, point2, point3, point4])

referenceMean = sample1.computeMean()
referenceVariance = sample1.computeVariance()
referenceSkewness = sample1.computeSkewness()
referenceKurtosis = sample1.computeKurtosis()

mixedSample = ot.Sample(sample1)
mixedSample.add(point1)
mixedSample.add(point2)
mixedSample.add(point3)
mixedSample.add(point4)
referenceMixedMean = mixedSample.computeMean()
referenceMixedVariance = mixedSample.computeVariance()
referenceMixedSkewness = mixedSample.computeSkewness()
referenceMixedKurtosis = mixedSample.computeKurtosis()

# Iterative moments, one point at a time
dimension = 2
order = 4
iterPoint = ot.IterativeMoments(order, dimension)
iterPoint.increment(point1)
iterPoint.increment(point2)
iterPoint.increment(point3)
iterPoint.increment(point4)

ott.assert_almost_equal(iterPoint.getMean(), referenceMean)
ott.assert_almost_equal(iterPoint.getVariance(), referenceVariance)
ott.assert_almost_equal(iterPoint.getSkewness(), referenceSkewness)
ott.assert_almost_equal(iterPoint.getKurtosis(), referenceKurtosis)
ott.assert_almost_equal(iterPoint.getDimension(), 2)
ott.assert_almost_equal(iterPoint.getOrder(), 4)
ott.assert_almost_equal(iterPoint.getIterationNumber(), 4)


# Iterative moments, one single sample
iterSample = ot.IterativeMoments(order, dimension)
iterSample.increment(sample1)
ott.assert_almost_equal(iterSample.getMean(), referenceMean)
ott.assert_almost_equal(iterSample.getVariance(), referenceVariance)
ott.assert_almost_equal(iterSample.getSkewness(), referenceSkewness)
ott.assert_almost_equal(iterSample.getKurtosis(), referenceKurtosis)
ott.assert_almost_equal(iterSample.getDimension(), 2)
ott.assert_almost_equal(iterSample.getOrder(), 4)
ott.assert_almost_equal(iterSample.getIterationNumber(), 4)


# Iterative moments, sample and points
iterMixed = ot.IterativeMoments(4, dimension)
iterMixed.increment(sample1)
iterMixed.increment(point1)
iterMixed.increment(point2)
iterMixed.increment(point3)
iterMixed.increment(point4)
ott.assert_almost_equal(iterMixed.getMean(), referenceMixedMean)
ott.assert_almost_equal(iterMixed.getVariance(), referenceMixedVariance)
ott.assert_almost_equal(iterMixed.getSkewness(), referenceMixedSkewness)
ott.assert_almost_equal(iterMixed.getKurtosis(), referenceMixedKurtosis)
ott.assert_almost_equal(iterMixed.getDimension(), 2)
ott.assert_almost_equal(iterMixed.getOrder(), 4)
ott.assert_almost_equal(iterMixed.getIterationNumber(), 8)



import openturns as ot
import openturns.viewer as otv

f1 = ot.SymbolicFunction(["x"], ["sin(x)"])
f2 = ot.SymbolicFunction(["x"], ["x"])

f = f1 * f2
graph = f.draw(0.0, 10.0)
graph.setTitle("y=f1*f2")
otv.View(graph, figure_kw={"figsize": (8, 4)}, add_legend=True)



#! /usr/bin/env python

import openturns as ot

ot.TESTPREAMBLE()


def clean(inPoint):
    dim = inPoint.getDimension()
    for i in range(dim):
        if abs(inPoint[i]) < 1.0e-10:
            inPoint[i] = 0.0
        else:
            inPoint[i] = 1e-4 * int(1e4 * inPoint[i])
    return inPoint


iMax = 5
# First test
distribution = ot.Uniform()
algo0 = ot.LegendreFactory()
algo1 = ot.AdaptiveStieltjesAlgorithm(distribution)
print(algo1)
for i in range(iMax):
    print(
        distribution.getClassName() + " Reference(",
        i,
        ")=",
        clean(algo0.getRecurrenceCoefficients(i)),
    )
    print(
        distribution.getClassName() + " AdaStielj(",
        i,
        ")=",
        clean(algo1.getRecurrenceCoefficients(i)),
    )

# Second test
distribution = ot.Beta(0.5, 2.5, -1.0, 1.0)
algo0 = ot.JacobiFactory(0.5, 3.0, 1)
algo1 = ot.AdaptiveStieltjesAlgorithm(distribution)
print(algo1)
for i in range(iMax):
    print(
        distribution.getClassName() + " Reference(",
        i,
        ")=",
        clean(algo0.getRecurrenceCoefficients(i)),
    )
    print(
        distribution.getClassName() + " AdaStielj(",
        i,
        ")=",
        clean(algo1.getRecurrenceCoefficients(i)),
    )



#! /usr/bin/env python

import openturns as ot
import openturns.testing as ott

ot.TESTPREAMBLE()

# Instantiate one distribution object
distribution = ot.Laplace(0.5, 1.5)
print("Distribution ", repr(distribution))
print("Distribution ", distribution)

# Is this distribution elliptical ?
print("Elliptical = ", distribution.isElliptical())

# Is this distribution continuous ?
print("Continuous = ", distribution.isContinuous())

# Test for realization of distribution
oneRealization = distribution.getRealization()
print("oneRealization=", repr(oneRealization))

# Define a point
point = ot.Point(distribution.getDimension(), 1.0)
print("Point= ", repr(point))

# Show PDF and CDF of point
DDF = distribution.computeDDF(point)
print("ddf     =", repr(DDF))

PDF = distribution.computePDF(point)
print("pdf     = %.10f" % PDF)
CDF = distribution.computeCDF(point)
print("cdf= %.12g" % CDF)
CF = distribution.computeCharacteristicFunction(point[0])
print("characteristic function= (%.12g%+.12gj)" % (CF.real, CF.imag))
PDFgr = distribution.computePDFGradient(point)
print("pdf gradient     =", repr(PDFgr))
CDFgr = distribution.computeCDFGradient(point)
print("cdf gradient     =", repr(CDFgr))
quantile = distribution.computeQuantile(0.95)
print("quantile=", repr(quantile))
print("cdf(quantile)=", distribution.computeCDF(quantile))
# Get 95% survival function
inverseSurvival = ot.Point(distribution.computeInverseSurvivalFunction(0.95))
print("InverseSurvival=", repr(inverseSurvival))
print(
    "Survival(inverseSurvival)=%.6f"
    % distribution.computeSurvivalFunction(inverseSurvival)
)
print("entropy=%.6f" % distribution.computeEntropy())

# Confidence regions
interval, threshold = distribution.computeMinimumVolumeIntervalWithMarginalProbability(
    0.95
)
print("Minimum volume interval=", interval)
print("threshold=", ot.Point(1, threshold))
levelSet, beta = distribution.computeMinimumVolumeLevelSetWithThreshold(0.95)
print("Minimum volume level set=", levelSet)
print("beta=", ot.Point(1, beta))
interval, beta = distribution.computeBilateralConfidenceIntervalWithMarginalProbability(
    0.95
)
print("Bilateral confidence interval=", interval)
print("beta=", ot.Point(1, beta))
(
    interval,
    beta,
) = distribution.computeUnilateralConfidenceIntervalWithMarginalProbability(0.95, False)
print("Unilateral confidence interval (lower tail)=", interval)
print("beta=", ot.Point(1, beta))
(
    interval,
    beta,
) = distribution.computeUnilateralConfidenceIntervalWithMarginalProbability(0.95, True)
print("Unilateral confidence interval (upper tail)=", interval)
print("beta=", ot.Point(1, beta))

mean = distribution.getMean()
print("mean=", repr(mean))
covariance = distribution.getCovariance()
print("covariance=", repr(covariance))
parameters = distribution.getParametersCollection()
print("parameters=", repr(parameters))
print("Standard representative=", distribution.getStandardRepresentative())

# Specific to this distribution
print("lambda=", distribution.getLambda())
mu = distribution.getMu()
print("mu=", mu)
standardDeviation = distribution.getStandardDeviation()
print("standard deviation=", repr(standardDeviation))
skewness = distribution.getSkewness()
print("skewness=", repr(skewness))
kurtosis = distribution.getKurtosis()
print("kurtosis=", repr(kurtosis))

ot.Log.Show(ot.Log.TRACE)
validation = ott.DistributionValidation(distribution)
validation.run()



#!/usr/bin/env python

import openturns as ot
import openturns.experimental as otexp
import openturns.testing as ott
from openturns.usecases import cantilever_beam
from openturns.usecases import stressed_beam
import math

# stressed_beam ##############################################

print(f"-- stressed_beam {'-' * 50}")
threshold = -1e6
sm = stressed_beam.AxialStressedBeam()
limitStateFunction = sm.model
R_dist = sm.distribution_R
F_dist = sm.distribution_F
distribution = sm.distribution
inputRandomVector = ot.RandomVector(distribution)
outputRandomVector = ot.CompositeRandomVector(limitStateFunction, inputRandomVector)
event = ot.ThresholdEvent(outputRandomVector, ot.Less(), threshold)
D = 0.02
G = R_dist - F_dist / (D**2 / 4 * math.pi)
pf_ref = G.computeCDF(threshold)
print(f"exact probability={pf_ref}")

# FORM analysis
optimAlgo = ot.Cobyla()
optimAlgo.setStartingPoint(distribution.getMean())
algoFORM = ot.FORM(optimAlgo, event)
algoFORM.run()
resultFORM = algoFORM.getResult()
alpha = resultFORM.getStandardSpaceDesignPoint()
print(f"alpha={alpha}")

# LineSampling
ot.RandomGenerator.SetSeed(0)
solver = ot.Brent(1e-3, 1e-3, 1e-3, 5)
rootStrategy = ot.MediumSafe(solver)
algo = otexp.LineSampling(event, alpha, rootStrategy)
algo.setMaximumCoefficientOfVariation(1e-3)
algo.setMaximumOuterSampling(1000)
algo.setStoreHistory(True)
algo.run()
result = algo.getResult()
print(str(algo)[:100])
print(result)
ott.assert_almost_equal(result.getProbabilityEstimate(), pf_ref)
assert algo.getStoreHistory()
assert len(algo.getAlphaHistory()) == algo.getMaximumOuterSampling(), "empty alphas"
assert len(algo.getRootValuesHistory()) > 0, "empty roots"
assert len(algo.getRootPointsHistory()) > 0, "empty roots"
assert algo.getAdaptiveImportantDirection(), "adaptive?"
assert algo.getSearchOppositeDirection(), "opposite?"
assert algo.getInitialAlpha() == alpha, "alpha?"
rootStrategy2 = algo.getRootStrategy()
algo.setRootStrategy(rootStrategy2)

# cantilever_beam ##############################################

print(f"-- cantilever_beam {'-' * 50}")

# model
cb = cantilever_beam.CantileverBeam()
distribution = cb.distribution
model = cb.model
vect = ot.RandomVector(distribution)
G = ot.CompositeRandomVector(model, vect)
event = ot.ThresholdEvent(G, ot.Greater(), 0.30)

# FORM for alpha
optimAlgo = ot.AbdoRackwitz()
optimAlgo.setStartingPoint(distribution.getMean())
optimAlgo.setMaximumCallsNumber(1000)
optimAlgo.setMaximumAbsoluteError(1.0e-10)
optimAlgo.setMaximumRelativeError(1.0e-10)
optimAlgo.setMaximumResidualError(1.0e-10)
optimAlgo.setMaximumConstraintError(1.0e-10)
algo = ot.FORM(optimAlgo, event)
algo.run()
result = algo.getResult()
alpha = result.getStandardSpaceDesignPoint()
print(f"alpha={alpha}")

# LineSampling
ot.RandomGenerator.SetSeed(0)
solver = ot.Brent(1e-3, 1e-3, 1e-3, 5)
rootStrategy = ot.MediumSafe(solver)
algo = otexp.LineSampling(event, alpha, rootStrategy)
algo.setMaximumOuterSampling(1000)
algo.setMaximumCoefficientOfVariation(5e-2)
algo.run()
result = algo.getResult()
print(result)
ott.assert_almost_equal(result.getProbabilityEstimate(), 4.57807e-07)

# two branch model ##############################################

print(f"-- 2 branch {'-' * 50}")

X = ot.RandomVector(ot.Normal(2))
g_twoBranch = ot.SymbolicFunction(
    ["x1", "x2"],
    [
        "min(5 + 0.1 * (x1 - x2)^2 - (x1 + x2) / sqrt(2), 5 + 0.1 * (x1 - x2)^2 + (x1 + x2) / sqrt(2))"
    ],
)
Y_twoBranch = ot.CompositeRandomVector(g_twoBranch, X)
threshold = 1.5
event_twoBranch = ot.ThresholdEvent(Y_twoBranch, ot.Less(), threshold)

g1 = ot.SymbolicFunction(["x1", "x2"], ["5 + 0.1 * (x1 - x2)^2 - (x1 + x2) / sqrt(2)"])
Y1 = ot.CompositeRandomVector(g1, X)
event1 = ot.ThresholdEvent(Y1, ot.Less(), threshold)

g2 = ot.SymbolicFunction(["x1", "x2"], ["5 + 0.1 * (x1 - x2)^2 + (x1 + x2) / sqrt(2)"])
Y2 = ot.CompositeRandomVector(g2, X)
event2 = ot.ThresholdEvent(Y2, ot.Less(), threshold)
unionEvent = ot.UnionEvent([event1, event2])

optimAlgo = ot.Cobyla()
optimAlgo.setStartingPoint(X.getMean())
algo = ot.FORM(optimAlgo, event_twoBranch)
algo.run()
result = algo.getResult()
alpha_twoBranch = result.getStandardSpaceDesignPoint()

# LineSampling / only one branch
ot.RandomGenerator.SetSeed(0)
solver = ot.Brent(1e-3, 1e-3, 1e-3, 5)
rootStrategy = ot.MediumSafe(solver)
algo = otexp.LineSampling(event_twoBranch, alpha_twoBranch, rootStrategy)
algo.setSearchOppositeDirection(False)
algo.setMaximumOuterSampling(1000)
algo.setMaximumCoefficientOfVariation(5e-2)
algo.run()
result = algo.getResult()
print(result)
ott.assert_almost_equal(result.getProbabilityEstimate(), 1.5e-4, 0.0, 1e-5)

# LineSampling
ot.RandomGenerator.SetSeed(0)
algo.setSearchOppositeDirection(True)
algo.setMaximumOuterSampling(1000)
algo.run()
result = algo.getResult()
print(result)
ott.assert_almost_equal(result.getProbabilityEstimate(), 3e-4, 0.0, 1e-5)

# LineSampling / system event / no adaptive alpha
algo = otexp.LineSampling(unionEvent, alpha_twoBranch, rootStrategy)
ot.RandomGenerator.SetSeed(0)
algo.setMaximumOuterSampling(1000)
algo.setAdaptiveImportantDirection(False)
algo.run()
result = algo.getResult()
print(result)
ott.assert_almost_equal(result.getProbabilityEstimate(), 3e-4, 0.0, 1e-5)
ot.ResourceMap.SetAsBool("LineSampling-DefaultAdaptiveImportantDirection", True)


# rosenbrock ##############################################

print(f"-- rosenbrock {'-' * 50}")
g = ot.SymbolicFunction(["x1", "x2"], ["100 * (x2 - x1^2)^2 + (x1 - 1)^2"])
X = ot.RandomVector(ot.Normal(2))
Y = ot.CompositeRandomVector(g, X)
threshold = 0.1
event = ot.ThresholdEvent(Y, ot.Less(), threshold)
###################################################

# FORM for  alpha
optimAlgo = ot.Cobyla()
optimAlgo.setStartingPoint(X.getMean())
algo = ot.FORM(optimAlgo, event)
algo.run()
result = algo.getResult()
alpha = result.getStandardSpaceDesignPoint()

# LineSampling
rootStrategy = ot.SafeAndSlow(ot.Brent(1e-3, 1e-3, 1e-3, 5), 3, 0.1)
ot.RandomGenerator.SetSeed(0)
algo = otexp.LineSampling(event, alpha, rootStrategy)
algo.setMaximumOuterSampling(3000)
algo.setMaximumCoefficientOfVariation(5e-2)
algo.run()
result = algo.getResult()
print(result)
ott.assert_almost_equal(result.getProbabilityEstimate(), 0.00110258, 0.0, 1e-4)

# complementary
event = ot.ThresholdEvent(Y, ot.Greater(), threshold)
ot.RandomGenerator.SetSeed(0)
algo = otexp.LineSampling(event, alpha, rootStrategy)
algo.setMaximumOuterSampling(3000)
algo.setMaximumCoefficientOfVariation(5e-2)
algo.run()
result = algo.getResult()
print(result)
ott.assert_almost_equal(result.getProbabilityEstimate(), 0.997749, 0.0, 1e-4)

# complementary + opposite
ot.RandomGenerator.SetSeed(0)
algo = otexp.LineSampling(event, alpha, rootStrategy)
algo.setMaximumOuterSampling(3000)
algo.setMaximumCoefficientOfVariation(5e-2)
algo.setSearchOppositeDirection(True)
algo.run()
result = algo.getResult()
print(result)
ott.assert_almost_equal(result.getProbabilityEstimate(), 0.997749, 0.0, 1e-4)

# paraboloid ##############################################

print(f"-- paraboloid {'-' * 50}")

# model
paraboloid = ot.SymbolicFunction(
    ["u1", "u2", "u3", "u4", "u5"], ["- u5 + u1^2 + u2^2 + u3^2 + u4^2"]
)
b = 3.5
db = 0.02
t = 0.0
dim = 5
U = ot.Normal(dim)
X = ot.RandomVector(U)
Y = ot.CompositeRandomVector(paraboloid, X)
E1 = ot.ThresholdEvent(Y, ot.Less(), -b)
E2 = ot.ThresholdEvent(Y, ot.Less(), -(b + db))
E3 = ot.ThresholdEvent(Y, ot.Greater(), -(b + db))
E4 = ot.IntersectionEvent([E1, E3])

# LineSampling, with step < db to find the roots
alpha = [0.0, 0.0, 0.0, 0.0, 1.0]
rootStrategy = ot.SafeAndSlow(ot.Brent(1e-3, 1e-3, 1e-3, 5), 8, 0.01)
ot.RandomGenerator.SetSeed(0)
algo = otexp.LineSampling(E4, alpha, rootStrategy)
algo.setMaximumOuterSampling(2000)
algo.setMaximumCoefficientOfVariation(5e-2)
algo.run()
result = algo.getResult()
print(result)
ott.assert_almost_equal(result.getProbabilityEstimate(), 2.103779e-07)



#! /usr/bin/env python

import openturns as ot
import matplotlib.pyplot as plt
from matplotlib.backends.backend_pdf import PdfPages
from openturns.viewer import View
import time

ot.Log.Show(ot.Log.INFO)

# Bounds are [0,1]^dimension
dimension = 2
bounds = ot.Interval(dimension)

nSimu = 10000

c2 = ot.SpaceFillingC2()
# Size of sample
size = 10

print("dimension=%d, size=%d" % (dimension, size))
for nSimu in [
    100,
    200,
    400,
    800,
    1600,
    3200,
    6400,
    12800,
    25600,
    51200,
    102400,
    204800,
    409600,
]:
    ot.RandomGenerator.SetSeed(0)
    # Factory: lhs generates
    lhsDesign = ot.LHSExperiment(
        ot.JointDistribution([ot.Uniform(0.0, 1.0)] * dimension), size
    )
    lhsDesign.setAlwaysShuffle(True)  # randomized

    mc = ot.MonteCarloLHS(lhsDesign, nSimu, c2)
    tic = time.time()
    design = mc.generate()
    result = mc.getResult()
    toc = time.time()
    print("%d %f %f" % (nSimu, result.getOptimalValue(), toc - tic))

pp = PdfPages("small_mc_OTLHS.pdf")
# plot criterion & save it
crit = result.drawHistoryCriterion()
fig = View(crit, plot_kwargs={"color": "blue"}).getFigure()
pp.savefig(fig)
plt.close(fig)

minDist = ot.SpaceFillingMinDist()

# Factory: lhs generates
lhsDesign = ot.LHSExperiment(
    ot.JointDistribution([ot.Uniform(0.0, 1.0)] * dimension), size
)
lhsDesign.setAlwaysShuffle(True)  # randomized

mc = ot.MonteCarloLHS(lhsDesign, nSimu, minDist)
tic = time.time()
design = mc.generate()
result = mc.getResult()
toc = time.time()
print("cpu time=%f" % (toc - tic))
print("dimension=%d, size=%d,mc=%s" % (dimension, size, mc))
print(
    "optimal value="
    + str(result.getOptimalValue())
    + " c2="
    + str(result.getC2())
    + " phiP="
    + str(result.getPhiP())
    + " minDist="
    + str(result.getMinDist())
)
# plot criterion & save it
crit = result.drawHistoryCriterion()
fig = View(crit, plot_kwargs={"color": "blue"}).getFigure()
pp.savefig(fig)
plt.close(fig)

pp.close()



"""
Test Normality
==============
"""

# %%
# Normal fitting test using the Henry line
# ----------------------------------------
#
# In this paragraph we perform a visual goodness-of-fit test for a univariate
# Normal distribution using the Henry line test, which is the QQ plot adapted
# for Gaussian distributions.

# %%
import openturns as ot
import openturns.viewer as otv


# %%
# We first create the data :
distribution = ot.Normal(2.0, 0.5)
sample1 = distribution.getSample(100)

# %%
# We draw the Henry line plot and expect a good fitting :
graph = ot.VisualTest.DrawHenryLine(sample1)
view = otv.View(graph)

# %%
# For comparison sake e draw the Henry line plot for a Beta distribution. The result is expected to be bad.
sample2 = ot.Beta(0.7, 0.9, 0.0, 2.0).getSample(100)
graph = ot.VisualTest.DrawHenryLine(sample2)
view = otv.View(graph)


# %%
# Normality tests
# ---------------
#
# We use two tests to check whether a sample follows a Normal distribution :
#
#  - the Anderson-Darling test
#  - the Cramer-Von Mises test
#

# %%
# We first generate two samples, one from a standard unit Gaussian and another from a Gumbel
# distribution with parameters :math:`\beta = 1` and :math:`\gamma = 0`.
sample1 = ot.Normal().getSample(200)
sample2 = ot.Gumbel().getSample(200)

# %%
# We test the normality of the sample. We can display the result of the test as a yes/no answer with
# the `getBinaryQualityMeasure`. We can retrieve the p-value and the threshold with the `getPValue`
# and `getThreshold` methods.

# %%
test_result = ot.NormalityTest.AndersonDarlingNormal(sample1)
print(
    "Component is normal?",
    test_result.getBinaryQualityMeasure(),
    "p-value=%.6g" % test_result.getPValue(),
    "threshold=%.6g" % test_result.getThreshold(),
)

# %%
test_result = ot.NormalityTest.AndersonDarlingNormal(sample2)
print(
    "Component is normal?",
    test_result.getBinaryQualityMeasure(),
    "p-value=%.6g" % test_result.getPValue(),
    "threshold=%.6g" % test_result.getThreshold(),
)

# %%
test_result = ot.NormalityTest.CramerVonMisesNormal(sample1)
print(
    "Component is normal?",
    test_result.getBinaryQualityMeasure(),
    "p-value=%.6g" % test_result.getPValue(),
    "threshold=%.6g" % test_result.getThreshold(),
)

# %%
test_result = ot.NormalityTest.CramerVonMisesNormal(sample2)
print(
    "Component is normal?",
    test_result.getBinaryQualityMeasure(),
    "p-value=%.6g" % test_result.getPValue(),
    "threshold=%.6g" % test_result.getThreshold(),
)

# %%
# Display all figures
otv.View.ShowAll()



#! /usr/bin/env python

import importlib
import openturns as ot


def try_import(name):
    try:
        importlib.import_module(name)
        return True
    except ImportError:
        return False


print(ot.PlatformInfo.Summary())

for module in ["openturns.viewer", "dill", "psutil"]:
    status = "ok" if try_import(module) else "n/a"
    print(f"{module:<20}{status}")



#! /usr/bin/env python

import openturns as ot

ot.TESTPREAMBLE()

inputDimension = 3

myFunction = ot.TranslationFunction([2.0, 3.0, -4.0])
myFunction.setName("translationFunction")
inPoint = ot.Point(inputDimension)
inPoint[0] = 7.0
inPoint[1] = 8.0
inPoint[2] = 9.0
outPoint = myFunction(inPoint)
print("myFunction=", repr(myFunction))
print(myFunction.getName(), "( ", repr(inPoint), " ) = ", repr(outPoint))
print(
    myFunction.getName(),
    ".gradient( ",
    repr(inPoint),
    " ) = ",
    repr(myFunction.gradient(inPoint)),
)
print(
    myFunction.getName(),
    ".hessian( ",
    repr(inPoint),
    " ) = ",
    repr(myFunction.hessian(inPoint)),
)



#! /usr/bin/env python

import openturns as ot

ot.TESTPREAMBLE()

dim = 3
mean = ot.Point(dim)
sigma = ot.Point(dim)
R = ot.CorrelationMatrix(dim)
for i in range(dim):
    mean[i] = i + 0.5
    sigma[i] = 2 * i + 1.0
    for j in range(i):
        R[i, j] = 0.5 * (1.0 + i) / dim
distribution = ot.Normal(mean, sigma, R)
size = 10000
sample = distribution.getSample(size)
factory = ot.NormalFactory()
estimatedDistribution = factory.build(sample)
print("distribution=", repr(distribution))
print("Estimated distribution=", repr(estimatedDistribution))
estimatedDistribution = factory.build()
print("Default distribution=", estimatedDistribution)
estimatedDistribution = factory.build(distribution.getParameter())
print("Distribution from parameters=", estimatedDistribution)
estimatedNormal = factory.buildAsNormal(sample)
print("Normal          =", distribution)
print("Estimated normal=", estimatedNormal)
estimatedNormal = factory.buildAsNormal()
print("Default normal=", estimatedNormal)
estimatedNormal = factory.buildAsNormal(distribution.getParameter())
print("Normal from parameters=", estimatedNormal)
# 1D estimation
result = factory.buildEstimator(sample[:, 0])
estimatedDistribution = result.getDistribution()
print("Estimated distribution            =", repr(estimatedDistribution))
parameterDistribution = result.getParameterDistribution()
print("Parameter distribution            =", parameterDistribution)
# Robust estimation
factory = ot.NormalFactory(True)
estimatedDistribution = factory.build(sample)
print("distribution=", distribution)
print("Estimated distribution=", estimatedDistribution)



"""
Create a compound distribution
===================================
"""

# %%
# In this example we are going to build the distribution of the random vector :math:`\inputRV`
# defined by the conditional distribution of:
#
# .. math::
#
#    \inputRV|\vect{\Theta}
#
# where :math:`\vect{\Theta}` is the output of the random variable :math:`\vect{Y}` through the link
# function :math:`f`:
#
# .. math::
#
#    \vect{\Theta} & = f(\vect{Y})\\
#    \vect{Y} & \sim \cL_{\vect{Y}}
#
# This example creates a :class:`~openturns.CompoundDistribution` which offers all the methods
# attached to the distributions.
#
# We consider the case where :math:`X` is of dimension 1 and follows a uniform distribution defined
# by:
#
# ===============  =========================================================  ===============================
# Variable         Distribution                                               Parameter
# ===============  =========================================================  ===============================
# :math:`X`        :class:`~openturns.Uniform` (:math:`a, b`)                 :math:`(a,b) = (Y, 1+Y^2)`
# :math:`Y`        :class:`~openturns.Uniform` (:math:`c, d`)                 :math:`(c,d) = (-1, 1)`
# ===============  =========================================================  ===============================
#

# %%
import openturns as ot
import openturns.viewer as otv

# %%
# Create the :math:`Y` distribution.
YDist = ot.Uniform(-1.0, 1.0)

# %%
# Create the link function :math:`f: y \rightarrow (y, 1+y^2)`.
f = ot.SymbolicFunction(["y"], ["y", "1+y^2"])

# %%
# Create the conditional distribution of :math:`\vect{X}|\vect{\Theta}`: as the parameters have no
# importance, we use the default distribution.
XgivenThetaDist = ot.Uniform()

# %%
# In this example, the range of :math:`\vect{X}` depends on its parameters which are random. Thus, the model is not regular.
# We advice to change the discretization method that performs the integration.
# By default, this method is the quadrature method *GaussProduct*. We prefer to use the *QMC*.
# We use the :class:`~openturns.ResourceMap`.
ot.ResourceMap.SetAsString("CompoundDistribution-ContinuousDiscretizationMethod", "QMC")

# %%
# Create the compound distribution of :math:`X`.
XDist = ot.CompoundDistribution(XgivenThetaDist, YDist, f)
XDist.setDescription([r"$X|\mathbf{\Theta} = f(Y)$"])
XDist

# %%
# Get a sample:
XDist.getSample(5)

# %%
# Draw the PDF.
graph = XDist.drawPDF()
view = otv.View(graph)


# %%
view.ShowAll()



#! /usr/bin/env python

import openturns as ot

ot.TESTPREAMBLE()


# TEST NUMBER ZERO : DEFAULT CONSTRUCTOR AND STRING CONVERTER
print("test number zero : default constructor and string converter")

# Default constructor
symmetricTensor0 = ot.SymmetricTensor()

# String converter
print("symmetricTensor0 = ", repr(symmetricTensor0))

# TEST NUMBER ONE : CONSTRUCTOR WITH SIZE, OPERATOR() AND STRING CONVERTER
print("test number one : constructor with size, operator() and string converter")

# Constructor with size
symmetricTensor1 = ot.SymmetricTensor(2, 3)

# Check operator() methods
symmetricTensor1[0, 0, 0] = 1.0
symmetricTensor1[1, 0, 0] = 2.0
symmetricTensor1[1, 1, 0] = 3.0
symmetricTensor1[0, 0, 1] = 4.0
symmetricTensor1[1, 0, 1] = 5.0
symmetricTensor1[1, 1, 1] = 6.0
symmetricTensor1[0, 0, 2] = 7.0
symmetricTensor1[1, 0, 2] = 8.0
symmetricTensor1[1, 1, 2] = 9.0

# String converter
print("symmetricTensor1 = ", repr(symmetricTensor1))
print(
    "values = %.6f" % symmetricTensor1[0, 0, 0],
    "  %.6f" % symmetricTensor1[0, 1, 0],
    "  %.6f" % symmetricTensor1[1, 0, 0],
    "  %.6f" % symmetricTensor1[1, 1, 0],
    "  %.6f" % symmetricTensor1[0, 0, 1],
    "  %.6f" % symmetricTensor1[0, 1, 1],
    "  %.6f" % symmetricTensor1[1, 0, 1],
    "  %.6f" % symmetricTensor1[1, 1, 1],
    "  %.6f" % symmetricTensor1[0, 0, 2],
    "  %.6f" % symmetricTensor1[0, 1, 2],
    "  %.6f" % symmetricTensor1[1, 0, 2],
    "  %.6f" % symmetricTensor1[1, 1, 2],
)

# TEST NUMBER TWO : COPY CONSTRUCTOR AND STRING CONVERTER
print("test number two : copy constructor and string converter")

# Copy constructor
symmetricTensor2 = ot.SymmetricTensor(symmetricTensor1)

# String converter
print("symmetricTensor2 = ", repr(symmetricTensor2))

# TEST NUMBER THREE : GET DIMENSIONS METHODS
print("test number three : get dimensions methods")

# Get dimension methods
print("symmetricTensor1's nbRows = ", symmetricTensor1.getNbRows())
print("symmetricTensor1's nbColumns = ", symmetricTensor1.getNbColumns())
print("symmetricTensor1's nbSheets = ", symmetricTensor1.getNbSheets())

# TEST NUMBER FOUR : ASSIGNMENT METHOD
print("test number four : assignment method")
# no sense with python

# TEST NUMBER FIVE : ISEMPTY METHOD
print("test number five : isEmpty method")

# Check method isEmpty
symmetricTensor5 = ot.SymmetricTensor()
symmetricTensor6 = ot.SymmetricTensor()
print("symmetricTensor1 is empty = ", symmetricTensor1.isEmpty())
print("symmetricTensor0 is empty = ", symmetricTensor0.isEmpty())
print("symmetricTensor5 is empty = ", symmetricTensor5.isEmpty())
print("symmetricTensor6 is empty = ", symmetricTensor6.isEmpty())

# TEST NUMBER SIX : GETSHEET AND SETSHEET METHODS
print("symmetricTensor1 = ", repr(symmetricTensor1))

sheet1 = symmetricTensor1.getSheet(1)
print("symmetricTensor1.getSheet(1) = ", repr(sheet1))

sheet2 = ot.SymmetricMatrix(2)
sheet2[0, 0] = 0.4
sheet2[1, 0] = 0.5
sheet2[1, 1] = 0.6
print("sheet2 = ", repr(sheet2))
symmetricTensor1.setSheet(1, sheet2)
print("symmetricTensor1 = ", repr(symmetricTensor1))



"""
Estimate moments from sample
============================
"""

# %%
# In this example we are going to estimate statistical moments from a sample, eventually from an output variable of interest.

# %%
import openturns as ot


# %%
# We first define the model, the input distribution and the output distribution.

# model f
model = ot.SymbolicFunction(["x1", "x2"], ["x1^2+x2", "x2^2+x1"])

# input vector X
inputDist = ot.JointDistribution([ot.Normal()] * 2, ot.IndependentCopula(2))
inputDist.setDescription(["X1", "X2"])
inputVector = ot.RandomVector(inputDist)

# output vector Y=f(X)
output = ot.CompositeRandomVector(model, inputVector)

# sample Y
size = 1000
sample = output.getSample(size)

# %%
# Estimate mean
sample.computeMean()

# %%
# Estimate standard deviation
sample.computeStandardDeviation()

# %%
# Estimate variance
sample.computeVariance()

# %%
# Estimate skewness
sample.computeSkewness()

# %%
# Estimate kurtosis
sample.computeKurtosis()



"""
Create a domain event
=====================
"""

# %%
# Abstract
# --------
#
# We present in this example the creation and the use of a :class:`~openturns.DomainEvent` through a
# simple Monte-Carlo estimator.
#
import openturns as ot
import openturns.viewer as otv

# %%
# We consider a standard unit Gaussian bivariate random vector :math:`\vect{X} = (X_1,X_2)` with
# independent marginals.
dim = 2
distX = ot.Normal(dim)

# %%
# We define a model :math:`f` which maps a vector of :math:`\mathbb{R}^2` to another vector of :math:`\mathbb{R}^2`
#
# .. math::
#
#    f : (x_1, x_2) \mapsto (x_1 + x_2, 2x_1)
#
#
f = ot.SymbolicFunction(["x1", "x2"], ["x1+x2", "2*x1"])


# %%
# We build a :class:`~openturns.RandomVector` out of the input distribution and
# a :class:`~openturns.CompositeRandomVector` by using the model.
vecX = ot.RandomVector(distX)
vecY = ot.CompositeRandomVector(f, vecX)


# %%
# Definition and vizualisation of a domain event
# ----------------------------------------------
#

# %%
# We define for each marginals of the output random vector `vecY` a domain of interest, say :math:`[0,1] \times [0,1]`
domain = ot.Interval([0.0, 0.0], [1.0, 1.0])

# %%
# The :class:`~openturns.DomainEvent` is then built from the output random vector `vecY` and the `domain` :
event = ot.DomainEvent(vecY, domain)

# %%
# This domain is
#
# .. math::
#
#    \mathcal{D} = \{ \vect{x}=(x_1, x_2) \in \mathbb{R}^2 \; | \; x_1+x_2 \in [0,1] \; \mathrm{and} \; 2x_1 \in [0,1] \}.
#
#

# %%
# We plot both marginals of the model and the domain of interest for each marginal using contour curves.
#

# %%
# We  represent the first marginal of `vecY`.
ot.ResourceMap.SetAsUnsignedInteger("Contour-DefaultLevelsNumber", 7)
graphModel0 = f.draw(0, 1, 0, [0.0, 0.0], [-5.0, -5.0], [5.0, 5.0])
graphModel0.setXTitle(r"$x_1$")
graphModel0.setYTitle(r"$x_2$")
graphModel0.setTitle(r"Isolines of the model : $Y = f(X)$, first marginal")

# %%
# We represent the second marginal of `vecY`.
graphModel1 = f.draw(0, 1, 1, [0.0, 0.0], [-5.0, -5.0], [5.0, 5.0])
graphModel1.setXTitle(r"$x_1$")
graphModel1.setYTitle(r"$x_2$")
graphModel1.setTitle(r"Isolines of the model : $Y = f(X)$, second marginal")

# %%
# We shall now represent the curves delimiting the domain of interest :
#
nx, ny = 15, 15
xx = ot.Box([nx], ot.Interval([-5.0], [5.0])).generate()
yy = ot.Box([ny], ot.Interval([-5.0], [5.0])).generate()
inputData = ot.Box([nx, ny], ot.Interval([-5.0, -5.0], [5.0, 5.0])).generate()
outputData = f(inputData)

# %%
# The contour line associated with the 0.0 value for the first marginal.
mycontour0 = ot.Contour(xx, yy, outputData.getMarginal(0))
mycontour0.setLevels([0.0])
mycontour0.setLabels(["0.0"])
mycontour0.setColor("black")
mycontour0.setLineStyle("dashed")
graphModel0.add(mycontour0)

# %%
# The contour line associated with the 1.0 value for the first marginal.
mycontour1 = ot.Contour(xx, yy, outputData.getMarginal(0))
mycontour1.setLevels([1.0])
mycontour1.setLabels(["1.0"])
mycontour1.setColor("black")
mycontour1.setLineStyle("dashed")
graphModel0.add(mycontour1)
view = otv.View(graphModel0)

# %%
# The contour line associated with the 0.0 value for the second marginal.
mycontour2 = ot.Contour(xx, yy, outputData.getMarginal(1))
mycontour2.setLevels([0.0])
mycontour2.setLabels(["0.0"])
mycontour2.setColor("black")
mycontour2.setLineStyle("dashed")
graphModel1.add(mycontour2)

# %%
# The contour line associated with the 1.0 value for the second marginal.
mycontour3 = ot.Contour(xx, yy, outputData.getMarginal(1))
mycontour3.setLevels([1.0])
mycontour3.setLabels(["1.0"])
mycontour3.setColor("black")
mycontour3.setLineStyle("dashed")
graphModel1.add(mycontour3)
view = otv.View(graphModel1)


# %%
# For each marginal the domain of interest is the area between the two black dashed curves.
# The domain event :math:`\mathcal{D}` is the intersection of these two areas.
# Here the intersection of both events is a parallelogram with the following
# vertices :
data = [[0.0, 0.0], [0.5, -0.5], [0.5, 0.5], [0.0, 1.0], [0.0, 0.0]]

# %%
# We create a polygon from these vertices with the :class:`~openturns.Polygon`
# class : that is our domain event.
myGraph = ot.Graph("Domain event", r"$x_1$", r"$x_2$")
myPolygon = ot.Polygon(data)
myPolygon.setColor("darkgray")
myPolygon.setEdgeColor("darkgray")
myGraph.add(myPolygon)

# Some annotation
texts = [
    r"$\mathcal{D} = \{ \mathbf{x}=(x_1, x_2) \in \mathbb{R}^2 \; | \; x_1+x_2 \in [0,1] \; \mathrm{and} \; 2x_1 \in [0,1] \}$"
]

myText = ot.Text([0.25], [0.0], texts)
myText.setTextSize(1)
myText.setColor("black")
myGraph.add(myText)
view = otv.View(myGraph)


# %%
# An example
# ----------
#
# Consider the integral
#
# .. math::
#
#    P_f = \int_{\mathcal{D}} \mathbf{1}_{\mathcal{D}}(\vect{x}) f_{X_1,X_2}(\vect{x}) d \vect{x}
#
# where :math:`{\mathcal{D}}` is the previous domain event, :math:`\mathbf{1}_{\mathcal{D}}` is the indicator function on the domain
# and :math:`f_{X_1,X_2}` is the probability density function of the input variable.

# %%
# We observe the integration domain :math:`{\mathcal{D}}` superimposed on the 2D-PDF.
graphPDF = distX.drawPDF([-5.0, -5.0], [5.0, 5.0])
graphPDF.setXTitle(r"$x_1$")
graphPDF.setYTitle(r"$x_2$")
graphPDF.setTitle(r"Isolines of the 2D-PDF")
graphPDF.add(myPolygon)
view = otv.View(graphPDF, contour_kw={"norm": "log"})

# %%
# We shall use a basic Monte Carlo algorithm using the domain event to estimate the probability.
#
algoMC = ot.ProbabilitySimulationAlgorithm(event)
algoMC.setMaximumOuterSampling(1000)
algoMC.setBlockSize(100)
algoMC.setMaximumCoefficientOfVariation(0.02)
algoMC.run()
print("Pf = %.4f" % algoMC.getResult().getProbabilityEstimate())


# %%
# We draw the convergence history :
graphConvergence = algoMC.drawProbabilityConvergence()
view = otv.View(graphConvergence)

# %%
# We can use the :meth:`~openturns.DomainEvent.getSample` method of the event to estimate the probability :math:`P_f`.
# This method draws realizations of the underlying random input vector `vecX` and returns `True` if the corresponding output random vector is in the domain event.
# Then the ratio between the number of realizations in the domain and the total of realizations is a rough estimate
# of the probability :math:`P_f` which we compare with the previous Monte-Carlo estimator.
N = 30000
samples = event.getSample(N)
print("Basic estimator : %.4f" % (sum(samples)[0] / N))


# %%
# Display all figures
otv.View.ShowAll()



#! /usr/bin/env python

import openturns as ot

ot.TESTPREAMBLE()
ot.PlatformInfo.SetNumericalPrecision(5)

m = 10
x = [[0.5 + i] for i in range(m)]

inVars = ["a", "b", "c", "x"]
formulas = ["a + b * exp(c * x)", "(a * x^2 + b) / (c + x^2)"]
g = ot.SymbolicFunction(inVars, formulas)
trueParameter = [2.8, 1.2, 0.5]
params = [0, 1, 2]
model = ot.ParametricFunction(g, params, trueParameter)
y = model(x)
y += ot.Normal([0.0] * 2, [0.05] * 2, ot.IdentityMatrix(2)).getSample(m)
candidate = [1.0] * 3
methods = ["SVD", "QR", "Cholesky"]
for method in methods:
    print("method=", method)
    algo = ot.LinearLeastSquaresCalibration(model, x, y, candidate, method)
    algo.run()
    result = algo.getResult()
    print("result=", result)
    # Draw result
    graph = result.drawParameterDistributions()
    graph = result.drawResiduals()
    graph = result.drawObservationsVsInputs()
    graph = result.drawObservationsVsPredictions()



#! /usr/bin/env python

import openturns as ot

ot.TESTPREAMBLE()

distribution = ot.Normal(4)
size = 10
weightingDistribution = ot.Normal(distribution)
weightingDistribution.setMu(ot.Point(4, 1.0))
experiment = ot.ImportanceSamplingExperiment(distribution, weightingDistribution, size)
print("experiment = ", experiment)
sample, weights = experiment.generateWithWeights()
print("sample = ", repr(sample))
print("weights = ", repr(weights))
